from suplexedlibrary.types.goat.AlgoliaResponse import GoatAlgoliaResponse
from suplexedlibrary.database.postgres.postgres import get_session
import cloudscraper
import random
from colorama import Fore
from requests import Session, session
from pprint import pprint
from suplexedlibrary.model.GoatAccount import GoatAccount
from suplexedlibrary.database.mongo import nosql
from suplexedlibrary.proxies.provider import get_sync_data_center_proxy, get_sync_cheap_proxy, get_luminati_sync_static_residential, get_cloudscraper_datacenter_session, get_cloudscraper_static_residential_session
from suplexedlibrary.exceptions import CreateListingException, DeleteListingNotFoundException, GoatException400, GoatException404, GoatException500, UpdatePriceException, NoListingFoundForUpdateException, FailedFetchingListings, DeleteListingFailedException, Scraping403Exception, DeleteFailedListingAlreadySoldExceiption
from suplexedlibrary.useragents.user_agent_service import get_random_user_agent
from algoliasearch.search_client import SearchClient

class GoatApi:
    product_pricing_cache = {}
    requests_made = 0

    asks = []
    wants = []

    def __init__(
        self,
        user_account: GoatAccount = None,
        skip_login: bool = True
    ):
        if user_account == None:
            print(f'empty construction of {__name__}')
            return

        self.sess = get_cloudscraper_datacenter_session()
        # Generate the cloudscraper session
        self.scraper = get_cloudscraper_datacenter_session()

        if skip_login:
            return

        self.local_cache = {}
        self.local_database_cache = {}
        # self.user_agent = 'GOAT/2.28.3 (iPhone; iOS 13.3.1; Scale/3.00) Locale/en'
        self.user_agent = get_random_user_agent()
        self.generate_default_headers()
        self.hydrate_db_template_id_cache()

        # Login details below
        self.username = user_account.email
        self.password = user_account.password
        self.user_id = user_account.user_id
        self.x_px_authorization = user_account.x_px_authorization
        self.authorization = user_account.authorization
        self.x_emb_st = user_account.x_emb_st
        self.cookie = user_account.cookie
        self.x_emb_id = user_account.x_emb_id
        # self.user_agent = user_account.user_agent
        self.x_px_original_token = user_account.x_px_original_token

        # Take the login details and create header objects
        self.generate_headers()


    def generate_default_headers(self):
        self.anonymous_get_headers = {
            "accept": "application/json",
            "accept-language": "en-us",
            "accept-encoding": "br, gzip, deflate",
            "user-agent": self.user_agent,
            'referer': 'https://www.google.com'
        }

    def generate_headers(self):
        self.logged_in_headers = {
            'content-type': 'application/x-www-form-urlencoded; charset=utf-8',
            'x-px-authorization': '3',
            'accept': 'application/json',
            'authorization': self.authorization,
            'accept-language': 'en-us',
            'x-emb-st': self.x_emb_st,
            'accept-encoding': 'br, gzip, deflate',
            'content-length': '227',
            'user-agent': self.user_agent,
            'x-emb-id': self.x_emb_id,
            'x-px-original-token': self.x_px_original_token,
        }
        self.get_headers = {
            "user-agent": self.user_agent,
            "authorization": self.authorization,
            "accept-language": "en-us",
            # "x-px-authorization": self.x_px_authorization,
            # "accept": "application/json",
            # "x-emb-st": self.x_emb_st,
            # "x-emb-id": self.x_emb_id,
            # "accept-language": "en-us",
            # "accept-encoding": "br, gzip, deflate",
            #  'x-px-authorization': '3:b3432093cf8b73a44f15a5b3119bfa38c773dbbda9ff0f0693526763910bb4be:Z9w45Uqy6M5gglkZqCEcQtFtFZraHn0EeRpziZJvulk3wqoQHfF1le/cMjMLjNJFZiZoJ+dIlyGQTrOf4c/RMg==:1000:C9FMKvV2Mcf05K4wU0WC3tn0avuCB2BHX5z9VBaBdNHovLcq70vysRIcJlur5Qy33qx4k2lZSqG4hg1kjRwvWEjtjfVu6spx6LiikwcD9QK1/WbXyscvIswzqlH59AY70F1FMkPqaMTXT7NJloFzbjM6XNtqhi1bdGXbNTJWL/w='
        }
        self.delete_headers = {
            'user-agent': self.user_agent,
            'authorization': self.authorization,
            # 'x-px-authorization': self.x_px_authorization,
            # 'accept': 'application/json',
            # 'accept-language': 'en-us',
            # 'x-emb-st': self.x_emb_st,
            # 'accept-encoding': 'br, gzip, deflate',
            # 'content-length': '0',
            # 'x-emb-id': self.x_emb_id,
        }
        self.activate_want_headers = {
            'user-agent': self.user_agent,
            'authorization': self.authorization,
            'content-type': 'application/x-www-form-urlencoded; charset=utf-8',
            # 'x-px-authorization': self.x_px_authorization,
            # "accept": "application/json",
            # 'accept-language': 'en-us',
            # 'x-emb-st': self.x_emb_st,
            # 'accept-encoding': 'br, gzip, deflate',
            # 'content-length': '20',
            # 'x-emb-id': self.x_emb_id,
        }
        self.logged_in_headers['cookie'] = self.cookie
        self.get_headers['cookie'] = self.cookie
        self.delete_headers['cookie'] = self.cookie
        self.activate_want_headers['cookie'] = self.cookie

    def hydrate_db_template_id_cache(self):

        db = nosql.get_sneaker_db()
        template_id_cursor = db.template_id_cache.find()
        for template_id_object in template_id_cursor:
            self.local_database_cache[template_id_object['key']
                                      ] = template_id_object['template_id']

        db.client.close()

    def get_all_orders(self, limit: int = None):
        page = 1
        orders = []
        while True:
            url = f'https://www.goat.com/api/v1/orders?filter=buy&page={page}'
            print(url)

            try:
                res = self.scraper.get(url=url, headers=self.get_headers)
            except Exception as e:
                print(e)
                break

            if res.status_code != 200:
                print(f'{res.status_code} received while fetching all orders')

            res_json = res.json()
            if 'orders' not in res_json or not res_json['orders']:
                break
            orders += res_json['orders']
            page += 1

            if limit != None and page > limit:
                break

        return orders

    def get_all_sell_orders(self, limit: int = None):
        page = 1
        orders = []

        while True:
            url = f'https://www.goat.com/api/v1/orders?filter=sell&page={page}'
            print(url)

            try:
                res = self.sess.get(url=url, headers=self.get_headers)
            except Exception as e:
                print(e)
                break

            print(f'{res.status_code} getting goat data')
        
            res_json = res.json()
            if 'orders' not in res_json or not res_json['orders']:
                break
            orders += res_json['orders']
            page += 1

            if limit != None and page > limit:
                break

        return orders

    def get_product_template_id(self, goat_url: str) -> str:
        if goat_url == None:
            print('no goat url')
            return
        cache_key = f'goatProductTemplateId:{goat_url}'

        if cache_key in self.local_cache:
            print(f'Leveraging local cache for {goat_url} template id')
            return self.local_cache[cache_key]
        if cache_key in self.local_database_cache:
            print(f'Leveraging database cache for {goat_url} template id')
            return self.local_database_cache[cache_key]

        url = f'https://www.goat.com/api/v1/product_templates/{goat_url}/show_v2'
        if goat_url == None or goat_url == '':
            print('no template id')
        try:
            res = self.scraper.get(
                url=url,
                headers=self.anonymous_get_headers
            )
            res_json = res.json()
        except Exception as e:
            print(
                f'{e}: Exception occurred while fetching product template id for {goat_url}')
            return None

        if 'id' not in res_json:
            print(f'No product template id for {goat_url}')
            return None

        template_id = res_json['id']
        self.local_cache[cache_key] = template_id
        print(f'Success finding product template id for {goat_url}')

        db = nosql.get_sneaker_db()
        db.template_id_cache.update_one({'key': cache_key}, {
                                        '$set': {'key': cache_key, 'template_id': template_id}}, upsert=True)
        
        db.client.close()
        return template_id

    def get_goat_amount_cents(self, amount: int) -> int:
        return amount * 100

    @staticmethod
    def get_goat_amount_dollars(amount: int) -> float:
        return amount / 100

    def update_own(self, amount: int, slug: str, sell_floor: int = None):

            if sell_floor != None:
                if amount < sell_floor:
                    print("We tried to go below the sell floor for while updating a ask: " + str(
                        slug) + ', new_ask = ' + str(amount) + ' , sell_floor = ' + str(sell_floor))
                    return

            url = 'https://www.goat.com/api/v1/products/{}'.format(slug)
            payload = {
                'product[priceCents]': amount
            }
            print(payload)
            res = self.scraper.patch(
                url=url, headers=self.activate_want_headers, data=payload)

            if res.status_code == 200:
                print('\t[~] Success updating slug')
            elif res.status_code == 404:
                raise NoListingFoundForUpdateException
            else:
                print(res.content)
                print(f'\t[!] Failed updating price for slug: {res.status_code}')
                raise UpdatePriceException

    def create_new_own(self, inventory_id, amount: int, product_template_id: int, size: float, sell_floor: int = None) -> str:
        try:
            if sell_floor != None:
                if amount < sell_floor:
                    print("We tried to go below the sell floor for while placing a new ask: " + str(
                        inventory_id) + ', new_ask = ' + str(amount) + ' , sell_floor = ' + str(sell_floor))
                    return

            url = 'https://www.goat.com/api/v1/products'
            payload = {
                'product[userId]': self.user_id,
                'product[missingItems]': None,
                'product[priceCents]': amount,
                'product[saleStatus]': 'pending',
                'product[boxCondition]': 'good_condition',
                'product[hasScuffs]': 0,
                'product[hasDiscoloration]': 0,
                'product[hasTears]': 0,
                'product[hasDefects]': 0,
                'product[sizeUnit]': 'us',
                'product[productTemplateId]': product_template_id,
                'product[hasMissingInsoles]': 0,
                'product[size]': size,
                'product[shoeCondition]': 'new'
            }
            res = self.scraper.post(
                url=url, headers=self.activate_want_headers, data=payload)

            if res.status_code == 200:
                print('\t[+] Created new product ownership success')
                own_id = res.json()['id']
                self.batch_create(amount=amount, product_id=own_id)

                return own_id
            else:
                print('\t[!] Error creating new product ownership: ' +
                    str(res.status_code))
                print(res.json())
                raise Exception(f'Status code {res.status_code} received while trying to create a listing')
        
        except Exception as e:
            print(e)
            raise CreateListingException

    def batch_create(self, amount: int, product_id:  int):
        url = 'https://www.goat.com/api/v1/products/batch_create'
        payload = {
            'boxCondition': 'good_condition',
            'priceCents': amount,
            'productId': product_id
        }
        res = self.scraper.post(
            url=url, headers=self.activate_want_headers, data=payload)
        if res.status_code == 200:
            print('\t[+] Product batch create success')
        else:
            print('\t[!] Product batch create success: ' +
                  str(res.status_code))
            print(res.json())

    def delete_own(self, slug: str) -> str:
        headers = {
            'content-type': 'application/x-www-form-urlencoded; charset=utf-8',
            'x-px-authorization': self.x_px_authorization,
            'accept': 'application/json',
            'authorization': self.authorization,
            'accept-language': 'en-us',
            'x-emb-st': self.x_emb_st,
            'accept-encoding': 'br, gzip, deflate',
            'content-length': '0',
            'user-agent': self.user_agent,
            'x-emb-id': self.x_emb_id,
            'cookie': self.cookie
        }
        url = 'https://www.goat.com/api/v1/products/{}/deactivate'.format(slug)

        self.scraper.post(url=url, headers=headers)
        own_url = 'https://www.goat.com/api/v1/products/{}'.format(slug)
        res = self.scraper.delete(url=own_url, headers=headers)

        if res.status_code == 200:
            print(f'Success deleting {slug}')
            return slug
        elif res.status_code == 404:
            print(f'id not found')
            raise DeleteListingNotFoundException(res.status_code)
        elif res.status_code == 422:
            raise DeleteFailedListingAlreadySoldExceiption(res.status_code)            
        else:
            print(f'Error deleting {slug} with status_code: {res.status_code}')
            raise DeleteListingFailedException(res.status_code)

    def get_listings(self):
        current_page = 1
        headers = {
            **self.get_headers,
            'x-px-authorization': '3'
        }
        url = f'https://www.goat.com/api/v1/products?filter=time&includeMetadata=1&includeTotalCount=1&page={current_page}&userId={self.user_id}'
        res = self.scraper.get(url, headers=self.get_headers)
        if res.status_code != 200 and res.status_code != 302:
            print(f'exception occurred getting listings')
            raise FailedFetchingListings

        res_json = res.json()

        if not res_json['products']:
            return []

        total_pages = res_json['metadata']['totalPages']
        listings = res_json['products']
        for i in range(current_page + 1, total_pages + 1):
            url = f'https://www.goat.com/api/v1/products?filter=time&includeMetadata=1&includeTotalCount=1&page={i}&userId={self.user_id}'
            res = self.scraper.get(url, headers=self.get_headers)

            if res.status_code != 200 and res.status_code != 302:
                print(f'exception occurred getting listings')
                raise FailedFetchingListings

            res_json = res.json()
            listings += res_json['products']

        return listings

    def get_goat_storage_listings(self):
        page = 1
        goat_storage_list = []

        if len(self.wants) > 0:
            # print('\t using cached wants')
            return self.wants

        while True:
            url = f'https://www.goat.com/api/v1/products?filter=goat_storage&includeMetadata=1&includeTotalCount=1&page={page}&userId={self.user_id}'
            res = self.scraper.get(url=url, headers=self.get_headers)

            if res.status_code != 200:
                print(res.content)
                raise FailedFetchingListings

            res_json = res.json()

            if len(res_json['products']) == 0:
                break
            goat_storage_list += res_json['products']
            page += 1

        print(f'found {len(goat_storage_list)} insta shippable products')
        return goat_storage_list        

    def login(self, username: str, password: str):
        user_agent = 'GOAT/2.23.1 (iPhone; iOS 13.2; Scale/3.00) Locale/en'
        sign_in_url = 'https://www.goat.com/api/v1/users/sign_in'
        self.scraper.headers.update({
            'user-agent': user_agent
        })

        homepage_headers = {
            'user-agent': user_agent
        }
        homepage_res = self.scraper.get('https://goat.com', headers=homepage_headers)
        
        sign_in_headers = {
            # 'Accept-Encoding':  'gzip,deflate',
            # 'Connection':       'keep-alive',
            # 'Accept':           '*/*',
            # 'Accept-Language':  'en-us',
            'Token':            'token=""',
            'User-Agent':        user_agent,
            'content-type':      'application/x-www-form-urlencoded; charset=utf-8',
            # 'x-px-authorization': '3:b3432093cf8b73a44f15a5b3119bfa38c773dbbda9ff0f0693526763910bb4be:Z9w45Uqy6M5gglkZqCEcQtFtFZraHn0EeRpziZJvulk3wqoQHfF1le/cMjMLjNJFZiZoJ+dIlyGQTrOf4c/RMg==:1000:C9FMKvV2Mcf05K4wU0WC3tn0avuCB2BHX5z9VBaBdNHovLcq70vysRIcJlur5Qy33qx4k2lZSqG4hg1kjRwvWEjtjfVu6spx6LiikwcD9QK1/WbXyscvIswzqlH59AY70F1FMkPqaMTXT7NJloFzbjM6XNtqhi1bdGXbNTJWL/w='
        }
        sign_in_body = {
            'user[login]': username,
            'user[password]': password,
        }

        print(f'status of homepage is {homepage_res.status_code}')
        sign_in_response = self.scraper.post(sign_in_url, data=sign_in_body, headers=sign_in_headers)
        
        print(f'status of sign in for {username} is {sign_in_response.status_code}')

        if sign_in_response.status_code != 200:
            if sign_in_response.status_code == 422:
                print(f'invalid password = {password}')
                return { 'incorrectPassword': True }
            # print(sign_in_response.content)
            pprint(self.scraper.proxies)
            pprint(sign_in_headers)
            pprint(sign_in_response.json())
            raise Exception('Failed sign in')

        sign_in_response_json = sign_in_response.json()
        pprint(sign_in_response_json)

        auth_token = sign_in_response_json['authToken']
        user_id = sign_in_response_json['id']
        sneaker_session = self.scraper.cookies.get_dict()['_sneakers_session']

        return {
            'x_px_original_token' : '3:458966ed586675e0735d4438d36b529a74879b946803cbe04848a4bf100d7f8c:8KF9Vgw/aLnLVGq19vjrqxBgKKemKx0q4AZ3k1iZYVQwPlsDVZyz3CcxVsirnoJefWP2uz9Y1JsdcuLawqhezg==:1000:tZNZoYgrDKjFKa0u/tZ4GMhMeg/pNxzWiLLm0H1Vz8E8b0FBf5Nxn/n132qmEOq5ea9xKs/rTpBtOPjNbGMyCGQcAS+WAFdjifEtSOsHiGIpagsQM68E9TRxzVpxT0qMB1HCWtjZfLU6uf9t8GK8LqaPR/qn8t15RJLPhn7Lcnk=',
            'authorization'       : f'Token token="{auth_token}"',
            'cookie'              : f'_sneakers_session={sneaker_session}',
            'x_emb_id'            : '576A7B17604A4F139C6390BDF9866FBD',
            'user_agent'          : user_agent,
            'x_px_authorization'  : '4',
            'x_emb_st'            : '1593021972759',
            'user_id'             : user_id,
        }

    def get_seller_info(self):
        url = 'https://www.goat.com/api/v1/users/me'
        me_response = self.scraper.get(url, headers=self.get_headers)
        if me_response.status_code != 200:
            print(f'Error {me_response.status_code} received for {url}')
        me_json = me_response.json()
        return (1 - me_json['earningsRate'], me_json['sellerFeeCents'] / 100, me_json['sellerScore'])

    def test_goat_login(self):
        url = 'https://www.goat.com/api/v1/users/me'
        me_response = self.scraper.get(url, headers=self.get_headers)
        if me_response.status_code != 200:
            raise Exception(f'Status {me_response.status_code} received at {url}')

        # pprint(me_response.json())

    def confirm_goat_sale_order(self, order_id: str, status_action: str = 'seller_confirm'):
        url = f'https://www.goat.com/api/v1/orders/{order_id}/update_status'

        body = {
                'order[statusAction]': status_action
        }
        pprint(body)
        confirmed_res = self.scraper.put(
            url=url,
            headers=self.activate_want_headers,
            data=body
        )
        if confirmed_res.status_code == 404:
            raise GoatException404('Order not found')
        
        if confirmed_res.status_code == 400:
            print(confirmed_res.json())
            raise GoatException400('Illegal request')

        if confirmed_res.status_code == 500:
            raise GoatException500('Error occurred')

        if confirmed_res.status_code != 200:
            raise Exception(f'Unkown Error {confirmed_res.status_code} occurred')
        
        # self.generate_shipping_label(exchange_ask_id=order_id)
        return confirmed_res

    def generate_shipping_label(self, exchange_ask_id: str):
        url = f'https://sell-api.goat.com/api/v1/purchase-orders/{exchange_ask_id}/generate-shipping-label'
        headers = {
            'user-agent': self.user_agent,
            'authorization': self.authorization,
            'content-type': 'application/x-www-form-urlencoded; charset=utf-8',            
        }        
        payload = {
            'number': exchange_ask_id
        }
        confirmed_res =self.scraper.put(
            url, headers=headers, data=payload
        )
        print(url)
        pprint(confirmed_res.json())
        if confirmed_res.status_code == 404:
            raise GoatException404()
        
        if confirmed_res.status_code == 400:
            raise GoatException400()

        if confirmed_res.status_code == 500:
            raise GoatException500()

        if confirmed_res.status_code != 200:
            raise Exception()

        return confirmed_res        

    @staticmethod
    def generate_app_agent():
        print()

    @staticmethod
    def get_sales_history(template_id: str, shoe_size: str, token: str) -> dict:
        headers = {
            "user-agent": get_random_user_agent(),
            'authorization': token,
            "accept-language": "en-us",
            "accept-encoding": "br, gzip, deflate",
            "accept": "application/json",
        }
        url = f'https://www.goat.com/api/v1/product_templates/{template_id}/offer_data_v2?size={shoe_size}'
        # print(f'fetching sales at {url}')
        sales_history_resonse = get_cloudscraper_datacenter_session().get(url, headers=headers)

        if sales_history_resonse.status_code != 200:
            # 
            # print(sales_history_resonse.status_code)
            # print(sales_history_resonse)
            print(f'{Fore.RED}\tFailed getting sales history for {template_id} size {shoe_size} status={sales_history_resonse.status_code}{Fore.RESET}')
            return []
        else:
            print(f'{Fore.GREEN}\tSuccess getting sales history for {template_id} size {shoe_size} status={sales_history_resonse.status_code}{Fore.RESET}')
        return sales_history_resonse.json()['priceData']

    @staticmethod
    def get_goat_historical_sales(slug: str, shoe_size: str, token: str) -> dict:
        url = f'https://sell-api.goat.com/api/v1/analytics/products/{slug}/historical-sales?box_condition=1&shoe_condition=1&size={shoe_size}'
        headers = {
            "user-agent": get_random_user_agent(),
            'authorization': token,
            "accept-language": "en-us",
            "accept-encoding": "br, gzip, deflate",
            "accept": "application/json",
        }
        sales_history_resonse = get_cloudscraper_datacenter_session().get(url, headers=headers)


        if sales_history_resonse.status_code == 403:
            raise Scraping403Exception(403)
        if sales_history_resonse.status_code != 200:
            print(f'{Fore.RED}\tFailed getting sales history for {slug} size {shoe_size} status={sales_history_resonse.status_code}{Fore.RESET}')
            raise Exception(sales_history_resonse.status_code)
        else:
            print(f'{Fore.GREEN}\tSuccess getting sales history for {slug} size {shoe_size} status={sales_history_resonse.status_code}{Fore.RESET}')
        
        sale_json = sales_history_resonse.json()
        return {
            'weekly_all_time': sale_json['weekly_all_time'] if 'weekly_all_time' in sale_json else [],
            'daily_365': sale_json['daily_365'] if 'daily_365' in sale_json else []
        }

    @staticmethod
    def search_goat_algolia_typed(query: str):
        client = SearchClient.create('2FWOTDVM2O', 'ac96de6fef0e02bb95d433d8d5c7038a')
        index = client.init_index('product_variants_v2')

        res = index.search(query)

        return GoatAlgoliaResponse(**res)



if __name__ == '__main__':
    GoatApi.search_goat_algolia_typed('britney griner')