import requests
from pprint import pprint
from datetime import datetime, timedelta
import dateutil.parser as dp

class ShippoApi:
    tracking_cache = {}

    def __init__(self):
        self.shippo_token = 'shippo_live_bc9f019b2b645472d3e9721f3266bf7f861a18df'


    def get_tracking_status(self, tracking_id: str, carrier: str) -> dict:
        cache_key = f'{carrier}:{tracking_id}'

        if cache_key in self.tracking_cache:
            return self.tracking_cache[cache_key]

        url = f'https://api.goshippo.com/tracks/{carrier}/{tracking_id}'
        headers = { 'Authorization': f'ShippoToken {self.shippo_token}' }
        
        tracking_res = requests.get(url, headers=headers)
        tracking_json = tracking_res.json()


        self.tracking_cache[cache_key] = tracking_json
        return tracking_json



    def is_package_delivered(self, tracking_id: str, carrier: str) -> bool:
        tracking_data = self.get_tracking_status(tracking_id, carrier)
        if ('tracking_status' in tracking_data 
            and tracking_data['tracking_status'] 
            and 'status' in tracking_data['tracking_status'] 
            and tracking_data['tracking_status']['status'] == 'DELIVERED'):

            print(f'packaage {carrier}:{tracking_id} is delivered')
            return True

        return False

    def get_days_until_delivery(self, tracking_id: str, carrier: str) -> int:
        tracking_data = self.get_tracking_status(tracking_id, carrier)

        if 'eta' not in tracking_data or not tracking_data['eta']:
            print(f'no eta for {carrier}:{tracking_id}')
            return None

        eta = tracking_data['eta'] 
        delivery_date = dp.isoparse(eta).replace(tzinfo=None, hour=0, minute=0, second=0, microsecond=0)
        now = datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)
        days_until_delivery = (delivery_date - now).days

        print(f'{days_until_delivery} until delivery')
        pass



if __name__ == '__main__':
    api = ShippoApi()
    api.get_days_until_delivery('1Z527Y0Y0354725783', 'ups')