from pprint import pprint
from uuid import uuid4
from suplexedlibrary.database.mongo.nosql import get_sneaker_db


class StockxHeadersUtil():
    def __init__(self, customer_id: str, user_agent: str, bearer_token: str) -> None:
        self.customer_id = customer_id
        self.user_agent = user_agent
        self.bearer_token = bearer_token
        self.mongo_db = get_sneaker_db()


    def get_etag(self, request_url: str):
        print(f'getting etag for reguest_url:{request_url} customer_id: {self.customer_id}')
        return self.mongo_db.assistEtagStore.find_one({'request_url': request_url, 'customer_id': self.customer_id})

    def write_etag(self, request_url: str, headers: dict):
        try:
            self.mongo_db.assistEtagStore.update_one(
                {'request_url': request_url, 'customer_id': self.customer_id},
                {'$set': {'request_url': request_url, 'customer_id': self.customer_id, 'etag': headers['etag']}},
            upsert=True)
        except Exception as e:
            print(f'{str(e)} Failed writing etags for {request_url} customer_id:{self.customer_id}')

    def get_buying_headers(self, request_url: str = None) -> dict:
        # etag = self.get_etag(request_url=request_url)

        # if etag:
        #     etag = etag['etag']

        headers = {
                'authority': 'stockx.com',
                'method': 'GET',
                'path': f'/api/customers/{self.customer_id}/buying/current?sort=created_at&order=DESC&limit=20&page=1&currency=USD&country=US',
                'scheme': 'https',
                'accept': '*/*',
                'accept-language': 'en-US,en;q=0.9',
                'appos': 'web',
                'appversion': '0.1',
                'cache-control': 'no-cache',
                'pragma': 'no-cache',
                'referer': 'https://stockx.com/buying',
                'sec-fetch-mode': 'cors',
                'sec-fetch-site': 'same-origin',
                'user-agent': self.user_agent,
                'authorization': f'Bearer {self.bearer_token}',
                # 'If-None-Match': etag if etag else '*'
        }

        # if etag:
        #     headers['If-None-Match'] = etag

        return headers



    def get_selling_headers(self, request_url: str = None):
        # etag = self.get_etag(request_url=request_url)

        # if etag:
        #     etag = etag['etag']

        headers = {
            'accept': '*/*',
            'accept-language': 'en-US,en;q=0.9',
            'appos': 'web',
            'appversion': '0.1',
            'referer': 'https://stockx.com/selling',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'user-agent': self.user_agent,
            'authorization': f'Bearer {self.bearer_token}',
        }
        
        # if etag:
        #     headers['If-None-Match'] = etag

        return headers

    def get_selling_headers_gql(self, request_url: str = None):
        # etag = self.get_etag(request_url=request_url)

        # if etag:
        #     etag = etag['etag']

        headers = {
            'content-type': 'application/json',
            'accept': '*/*',
            'accept-language': 'en-US,en;q=0.9',
            'apollographql-client-name': 'web',
            'apollographql-client-version': '0.1',
            'appos': 'web',
            'appversion': '0.1',
            'referer': 'https://stockx.com/selling',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'user-agent': self.user_agent,
            'authorization': f'Bearer {self.bearer_token}'
        }

        # if etag:
        #     headers['If-None-Match'] = etag

        return headers
    
    def get_notifications_headers_gql(
        self,
        device_id: str,
    ) -> dict:
        return {
            'authority': 'stockx.com',
            'Host': 'gateway.stockx.com',
            # 'If-None-Match': 'W/"dc-N0ocKk60Kjdk04DcnOabeK+3dS4"',
            'X-PX-AUTHORIZATION': '3:12f61c83475e18a32f7ba4b27139107cfc336bf93feae12870348ce8ecb687a7:236BVhGM0zV3esMtZDwPUPFNHqlK4nFOvPw0nra5RQGbXE5y96XK0ecDpeScVR2yNzucFkyFCKnCkY2yUXIISA==:1000:J/nfz9a7Nhnh1UYGtGgoDGA8MvOlYhU7tdpq/sJs26fByAMn9HcP9DOCZGXVTuyh8zgR3WE7GW9zCn2bbNmjxyFd8Oxj7dXyp4IeBnhi4SMfsRfvuoDPKJMFAU8D0nSMcNhkHCEWQQuvu2uo+giHhNlAgkOdhoKVs3I+CKuitoKvsuTI9qji2hdnJrrtAUuJ2qv9jUuodwgksniuht719doN8RcOXxWLBSUw3yuHMLs=',
            'user-agent': 'StockX/32514 CFNetwork/1325.0.1 Darwin/21.1.0',
            'x-session-id': str(uuid4()),
            'Accept': '*/*',
            'x-device-id': device_id,
            # 'x-experiment-ids': '5838122065657856,5009182807883776,6091767046799360,5233882838401024,1107155018,6339791884976128,6157718993829888,6375104065896448,5928664562728960,5100000484851712,6140580732141568,4805376063111168,6509873311318016,5393979235434496,5952562603819008,5315062856220672,5540026146422784,6030614759276544,6511217984208896,6401795974823936,5129095528644608,5633988932730880,6499701834711040,5297008432709632,5930679250518016,5502161639243776,6584007496105984,5475604749156352,5871836789276672,4963999485591552,5277403148517376,4921876426063872,4710575376498688,6633249900134400,6256623479750656,4663967800688640,6519321866076160,6044297610526720,4812510150393856,5541675438833664,5126514169217024,4771339499864064,6265306812907520,6628203665686528,6593655460462592,5247931879653376,5483248171745280,5520916712652800,5810881833074688,5127918782906368,6445406040752128,6566422733783040,5944634681196544,5916087476420608,5619977264431104,6427832125095936,5428571061223424,6532027662598144,5719240199766016,5870793141387264,4731142779305984,5858272722419712',
            'apollographql-client-name': 'stockx-ios-prod',
            'apollographql-client-version': '5.6.6',
            'x-abtest-ids': 'ab_all_in_ask_visibility_ios.true,ab_amount_entry_affirm_message_removal_ios.false,ab_amount_entry_fees_removal_ios.true,ab_checkout_ordered_failed_recovery_flow_ios.true,ab_checkout_review_order_button_text_ios.true,ab_enable_citcon_psp_ios.true,ab_enable_editing_order_review_ios.true,ab_enable_eu_vat_collection_ios.true,ab_enable_order_details_timeline_ios.true,ab_home_contentstack_modules_ios.variant,ab_ios_push_pre_prompt.variant_1,ab_new_bid_details_ios.false,ab_new_checkout_flow_v3_ios.true,ab_new_rfy_recipe_iosandroidweb.false,ab_pdp_urgency_badging_v2_ios.variant_1,ab_product_size_chart_ios.true',
            'authorization': f'Bearer {self.bearer_token}',
            'Accept-Language': 'en-US',
            'x-user-legacy-price-levels': 'legacy',
            'x-fraud-mobile-id': device_id,
            'App-Name': 'StockX-iOS',
            'Content-Type': 'application/json',
            'App-Version': '5.6.6.32451',
            'x-api-key': '99WtRZK6pS1Fqt8hXBfWq8BYQjErmwipa3a0hYxX',
            'x-drc-feature-localized-sizing': 'true',
            'App-Platform': 'ios'        
        }
    def get_delete_headers(self, request_url: str = None):
        # etag = self.get_etag(request_url=request_url)

        return {
            'content-type': 'application/json',
            'accept': '*/*',
            'accept-language': 'en-US,en;q=0.9',
            'apollographql-client-name': 'web',
            'apollographql-client-version': '0.1',
            'appos': 'web',
            'appversion': '0.1',
            'referer': 'https://stockx.com/selling',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'user-agent': self.user_agent,
            'authorization': f'Bearer {self.bearer_token}',         
            # 'If-None-Match': etag if etag else '*'   
        }


    def get_remove_order_headers(self, chain_id: str = None):
        # etag = self.get_etag(request_url=request_url)

        return {
            'authority': 'stockx.com',
            'method': 'DELETE',
            'path': f'/api/portfolio/{chain_id}',
            'scheme': 'https',
            'accept': '*/*',
            'accept-encoding': 'gzip, deflate, br',
            'accept-language': 'en-US,en;q=0.9',
            'appos': 'web',
            'appversion': '0.1',
            'authorization': f'Bearer {self.bearer_token}',
            'origin': 'https://stockx.com',
            'referer': 'https://stockx.com/selling',
            'sec-ch-ua': '" Not A;Brand";v="99", "Chromium";v="96", "Google Chrome";v="96"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"macOS"',
            'sec-fetch-dest': 'empty',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'user-agent': self.user_agent,
            'x-requested-with': 'XMLHttpRequest'
        }
        
    def get_update_bulk_ask_headers(self, request_url: str = None):
        # etag = self.get_etag(request_url=request_url)
        
        return {
            'authority': 'stockx.com',
            'method': 'POST',
            'path': '/api/portfolio?a=ask',
            'scheme': 'https',
            'accept': '*/*',
            'accept-language': 'en-US,en;q=0.9',
            'appos': 'web',
            'appversion': '0.1',
            'authorization': f'Bearer {self.bearer_token}',
            'content-length': '204',
            'content-type': 'application/json',
            'origin': 'https://stockx.com',
            'referer': 'https://stockx.com/sell/adidas-yeezy-boost-700-inertia',
            'sec-fetch-dest': 'empty',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'user-agent': self.user_agent,
            'x-requested-with': 'XMLHttpRequest',
            'x-session-id': '681a08d7-ef89-434a-999b-7d01a05267ca',       
            # 'If-None-Match': etag if etag else '*'   
        }
        
    def get_customer_id_headers(self):
        return {
            'user-agent': self.user_agent,
            'authorization': f'Bearer {self.bearer_token}',
            'authority': 'stockx.com'
        }
        
    def get_order_details_headers(self, request_url: str = None):
        # etag = self.get_etag(request_url=request_url)
        
        return {
            'authority': 'stockx.com',
            'method': 'GET',
            'scheme': 'https',
            'accept': '*/*',
            'accept-language': 'en-US,en;q=0.9',
            'appos': 'web',
            'appversion': '0.1',
            'cache-control': 'no-cache',
            'pragma': 'no-cache',
            'referer': 'https://stockx.com/selling',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'user-agent': self.user_agent,
            'authorization': f'Bearer {self.bearer_token}',         
            # 'If-None-Match': etag if etag else '*'
        }


    def get_seller_fees_headers(self, request_url: str = None):
        # etag = self.get_etag(request_url=request_url)

        return {
            'authority': 'stockx.com',
            'method': 'GET',
            'path': '/api/users/me/rewards',
            'scheme': 'https',
            'accept': '*/*',
            'accept-language': 'en-US,en;q=0.9',
            'appos': 'web',
            'appversion': '0.1',
            'authorization': f'Bearer {self.bearer_token}',
            'cache-control': 'no-cache',
            'pragma': 'no-cache',
            'referer': 'https://stockx.com/account',
            'sec-fetch-dest': 'empty',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'user-agent': self.user_agent,
            'x-requested-with': 'XMLHttpRequest',     
            # 'If-None-Match': etag if etag else '*'
        }
        

    def get_chain_id_headers(self, request_url: str = None):
        # etag = self.get_etag(request_url=request_url)
        
        return {
            'accept': '*/*',
            'accept-language': 'en-US,en;q=0.9',
            'appos': 'web',
            'appversion': '0.1',
            'referer': 'https://stockx.com/selling',
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'user-agent': self.user_agent,
            'authorization': f'Bearer {self.bearer_token}',    
            # 'If-None-Match': etag if etag else '*'
        }


    def get_pricing_headers(self, request_url: str = None):
        # etag = self.get_etag(request_url=request_url)

        return {
            'Content-Type': 'application/json',
            'App-Version': '3.7.3.12378',
            'x-api-key': '99WtRZK6pS1Fqt8hXBfWq8BYQjErmwipa3a0hYxX',
            'App-Name': 'StockX-iOS',
            'JWT-Authorization': '',
            'Accept-Language': 'en-us',
            'User-Agent': 'StockX/12378 CFNetwork/889.9 Darwin/17.2.0',         
            # 'If-None-Match': etag if etag else '*'          
        }        