class SyncSubscriptionPlan:
    name = ''
    description = ''
    price_id = ''

    listingLimit = 0
    syncLimit = 0
    pricingStrategyLimit = 0
    inventoryItemLimit = 0

    trialListingLimit = 0
    trialSyncLimit = 0
    trialPricingStrategyLimit = 0

    @classmethod
    def create_checkout(cls, customer_id: str, stripe: object):
        return stripe.checkout.Session.create(
            payment_method_types=['card'],
            line_items=[{
                'price': cls.price_id,
                'quantity': 1,
            }],
            customer=customer_id,
            mode='subscription',
            success_url='https://suplexed-sync-frontend.herokucom?session_id={CHECKOUT_SESSION_ID}',
            cancel_url='https://suplexed-sync-frontend.herokucom/'
        )

    @classmethod
    def create_free_trial_checkout(cls, customer_id: str, trialPeriod: int, stripe: object):
        return stripe.checkout.Session.create(
            payment_method_types=['card'],
            line_items=[{
                'price': cls.price_id,
                'quantity': 1,
            }],
            subscription_data={
                'trial_period_days': trialPeriod
            },
            customer=customer_id,
            mode='subscription',
            success_url='https://suplexed-sync-frontend.herokucom?session_id={CHECKOUT_SESSION_ID}',
            cancel_url='https://suplexed-sync-frontend.herokucom/'
        )

    def is_more_than_listing_limit(self, listing_count: int) -> bool:
        return self.listingLimit == listing_count

    def is_more_than_sync_limit(self, synced_count: int) -> bool:
        return self.syncLimit > synced_count

    def is_more_than_pricing_strategy_limit(self, pricing_strategy_count: int) -> bool:
        return self.pricingStrategyLimit > pricing_strategy_count

    def is_more_than_inventory_item_limit(self, inventory_item_count: int) -> bool:
        return self.inventoryItemLimit > inventory_item_count


class Anonymous(SyncSubscriptionPlan):
    price_id = None
    description = 'Anonymous'

    listingLimit = 0
    syncLimit = 0
    pricingStrategyLimit = 0

    trialListingLimit = 0
    trialSyncLimit = 0
    trialPricingStrategyLimit = 0


class Standard20201215(SyncSubscriptionPlan):
    price_id = 'price_1IPFJZG6XLeGzfOcHaGWdXyU'
    description = 'Standard'

    listingLimit = 5
    syncLimit = 5
    pricingStrategyLimit = 5

    trialListingLimit = 5
    trialSyncLimit = 5
    trialPricingStrategyLimit = 5


class Premium20201215(SyncSubscriptionPlan):
    price_id = 'price_1IPFHYG6XLeGzfOcq5XFzWjr' 
    description = 'Premium'

    listingLimit = 10
    syncLimit = 10
    pricingStrategyLimit = 10
    
    trialListingLimit = 1
    trialSyncLimit = 10
    trialPricingStrategyLimit = 10


def getSubscriptionPlanByPriceId(price_id: str):
    for cls in SyncSubscriptionPlan.__subclasses__():
        if cls.price_id == price_id:
            print(cls.description)
            return cls


if __name__ == '__main__':
    getSubscriptionPlanByPriceId('price_1IPFJZG6XLeGzfOcHaGWdXyU')