from sqlalchemy import create_engine, MetaData, Table, Column, Integer, String, ForeignKey, UniqueConstraint, Float, Date, Boolean, DateTime, TIMESTAMP, PickleType
from sqlalchemy.orm import relationship
from sqlalchemy.sql import func
from suplexedlibrary.model.Ask import Ask
from suplexedlibrary.model.AskGoat import AskGoat
from suplexedlibrary.model.AskStockX import AskStockX
from suplexedlibrary.model.DataSyncAccount import DataSyncAccount
from suplexedlibrary.model.GoatAccount import GoatAccount
from suplexedlibrary.model.InventoryItem import InventoryItem
from suplexedlibrary.model.PricingStrategy import PricingStrategy
from suplexedlibrary.model.ResellPricing import ResellPricing
from suplexedlibrary.model.StockxAccount import StockxAccount
from suplexedlibrary.model.SaleOrder import SaleOrder
from suplexedlibrary.model.PurchaseOrder import PurchaseOrder
from suplexedlibrary.model.SyncUserValueChain import SyncUserValueChain
from suplexedlibrary.model.AskStadiumGoods import AskStadiumGoods
from suplexedlibrary.model.FlightClubSellerScore import FlightClubSellerScore
from suplexedlibrary.model.ReportCardResult import ReportCardResult

def init_data_sync_account_table(meta: MetaData, engine):

    Table(
        'dataSyncAccount', meta,
        Column('id', String, primary_key=True),
        Column('first_name', String),
        Column('last_name', String),
        Column('date_of_birth', String),
        Column('email', String),
        Column('account_level', String),
        Column('registered_on', DateTime),
        Column('password_no_hash', String),
        Column('password_hash', String),
        Column('confirmed_email', Integer),
        UniqueConstraint('email', name='unique_email'),
    )

    meta.create_all(engine)


def init_goat_account_table(meta: MetaData, engine):
    Table('goatAccount', meta,
          Column('id', String, primary_key=True),
          Column('account_id', String, ForeignKey('dataSyncAccount.id')),
          Column('email', String),
          Column('password', String),
          Column('x_px_authorization', String),
          Column('authorization', String),
          Column('x_emb_st', String),
          Column('cookie', String),
          Column('x_emb_id', String),
          Column('user_agent', String),
          Column('x_px_original_token', String),
          Column('user_id', String),
          )

    meta.create_all(engine)


def init_ask_goat_table(meta: MetaData, engine):
    Table('askGoat', meta,
          Column('id', String, primary_key=True),
          Column('inventory_item_id', String, ForeignKey('inventoryItem.id')),
          Column('account_id', ForeignKey('dataSyncAccount.id')),
          Column('ask_amount', Float),
          Column('exchange', String),
          Column('exchange_ask_id', String),
          Column('style_id', String),
          Column('size', Float),
          UniqueConstraint('exchange_ask_id', name='unique_exchange_ask_id'),
          )

    meta.create_all(engine)


def init_ask_stockx_table(meta: MetaData, engine):
    Table('askStockX', meta,
          Column('id', String, primary_key=True),
          Column('inventory_item_id', String, ForeignKey('inventoryItem.id')),
          Column('account_id', ForeignKey('dataSyncAccount.id')),
          Column('ask_amount', Float),
          Column('exchange', String),
          Column('exchange_ask_id', String),
          Column('stx_sku_uuid', String),
          Column('style_id', String),
          Column('size', Float),
          UniqueConstraint('exchange_ask_id', name='unique_exchange_ask_id'),
          )

    meta.create_all(engine)


def init_inventory_item_table(meta: MetaData, engine):
    Table('inventoryItem', meta,
          Column('id', String, primary_key=True),
          Column('account_id', String, ForeignKey('dataSyncAccount.id')),
          Column('shoeName', String),
          Column('shoeSize', String),
          Column('shoeSku', String),
          Column('purchasePrice', Float),
          Column('purchaseFees', Float),
          Column('purchaseDiscount', Float),
          Column('purchaseShipping', Float),
          Column('purchaseDate', String),
          Column('email', String),
          Column('purchaseExchange', String),
          Column('soldDate', String),
          Column('soldPrice', Float),
          Column('soldFee', Float),
          Column('soldDiscount', Float),
          Column('soldShipping', Float),
          Column('soldExchange', String),
          Column('shoeGender', String),
          Column('purchaseAccount', String),
          Column('orderNumber', String),
          Column('orderStatus', String),
          Column('tracking', String),
          Column('purchase_offer_id', String)
          )

    meta.create_all(engine)


def init_pricing_strategy_table(meta: MetaData, engine):
    Table(
        'pricingStrategy', meta,
        Column('id', String, primary_key=True),
        Column('inventory_id', String, ForeignKey('inventoryItem.id')),
        Column('expiration_date', Date),
        Column('strategy_type', String),
        Column('amount', Integer),
        Column('maximize_profit', Boolean),
        Column('mute_goat', Boolean),
        Column('mute_stockx', Boolean),
    )

    meta.create_all(engine)


def init_resell_pricing_table(meta: MetaData, engine):
    Table(
        'resellPricing', meta,
        Column('id', String, primary_key=True),
        Column('inventory_id', String, ForeignKey('inventoryItem.id')),
        Column('goat_ask', Integer),
        Column('goat_offer', Integer),
        Column('stockx_ask', Integer),
        Column('stockx_offer', Integer),
        Column('stadium_goods_ask', Integer),
        Column('flight_club_ask', Integer),
        Column('time_created', DateTime(timezone=True),
               server_default=func.now()),
        Column('time_updated', DateTime(timezone=True), onupdate=func.now()),
        Column('stockx_ask_profit_margin', Float),
        Column('stockx_profit_margin', Float),
        Column('goat_ask_profit_margin', Float),
        Column('goat_profit_margin', Float),
    )

    meta.create_all(engine)


def init_stockx_account_table(meta: MetaData, engine):
    Table(
        'stockxAccount', meta,
        Column('id', String, primary_key=True),
        Column('account_id', String, ForeignKey('dataSyncAccount.id')),
        Column('email', String),
        Column('password', String),
        Column('customer_id', String),
    )

    meta.create_all(engine)


def init_sale_order_table(meta: MetaData, engine):
    Table(
        'saleOrder', meta,
        Column('id', String, primary_key=True),
        Column('inventory_id', String, ForeignKey('inventoryItem.id')),
        Column('account_id', String, ForeignKey('dataSyncAccount.id')),
        Column('marketplace', String),
        Column('shoe_sku', String),
        Column('shoe_name', String),
        Column('shoe_size', String),
        Column('exchange_ask_id', String),
        Column('sold_date', Date),
        Column('sold_amount', Float),
        Column('sold_fee', Float),
        Column('sold_shipping', Float),
        Column('sold_discount', Float),
        Column('image_url', String),

        Column('shipping_label_url', String),
        Column('is_sale_confirmed', Boolean),
        UniqueConstraint('exchange_ask_id', name='unique_exchange_ask_id'),
    )


def init_purchase_order_table(meta: MetaData, engine):
    Table(
        'purchaseOrder', meta,
        Column('id', String, primary_key=True),
        Column('inventory_id', String, ForeignKey('inventoryItem.id')),
        Column('account_id', String, ForeignKey('dataSyncAccount.id')),
        Column('marketplace', String),
        Column('shoe_sku', String),
        Column('shoe_name', String),
        Column('shoe_size', String),
        Column('exchange_ask_id', String),
        Column('purchase_date', Date),
        Column('purchase_amount', Float),
        Column('purchase_fee', Float),
        Column('purchase_shipping', Float),
        Column('purchase_discount', Float),
        Column('image_url', String),
        UniqueConstraint('exchange_ask_id', name='unique_exchange_ask_id'),
    )

def init_sync_user_value_chain(meta: MetaData, engine):
    Table(
        'syncUserValueChain', meta,
        Column('id', String, primary_key=True),
        Column('account_id', String, ForeignKey('inventoryItem.id')),
        Column('daily_listing_sync_count_stockx', Integer),
        Column('daily_listing_sync_count_goat', Integer),
        Column('daily_orders_sync_count_stockx', Integer),
        Column('daily_orders_sync_count_goat', Integer),
        Column('daily_listing_publish_count_stockx', Integer),
        Column('daily_listing_publish_count_goat', Integer),
    )

def init_daily_inventory_value_table(meta: MetaData, engine):
    Table(
        'dailyInventoryValue', meta,
        Column('id', String, primary_key=True),
        Column('account_id', String, ForeignKey('dataSyncAccount.id')),
        Column('date', DateTime),
        Column('value', Float),
    )

def init_stadium_goods_account_table(meta: MetaData, engine):
    Table(
        'stadiumGoodsAccount', meta,
        Column('id', String),
        Column('account_id', String),
        Column('email', String),
        Column('password', String),
        Column('cookies', PickleType),
    )

def init_inventory_sync_table(meta: MetaData, engine):
    Table(
        'inventorySync', meta,
        Column('id',                  String),
        Column('inventory_item_id',   String),
        Column('account_id',          String),
        Column('ask_amount',          Integer),
        Column('is_goat_active',      Boolean),
        Column('is_stockx_active',    Boolean),
    )


def init_seller_scores(meta, engine):
    Table(
        'goatSellerScore', meta,
        Column('id',                           String, primary_key=True),
        Column('account_id',                   String),
        Column('seller_flat_fee',              Integer),
        Column('seller_transaction_fee',       Float),
    )

    Table(
        'stockxSellerScore', meta,
        Column('id',                  String, primary_key=True),
        Column('account_id',          String),
        Column('email',               String),
        Column('password',            String),
        Column('customer_id',         String),
    )




def init_fc_account(meta, engine):
    Table(
        'flightClubAccount', meta,
        Column('id',                           String, primary_key=True),
        Column('account_id', String),
        Column('email', String),
        Column('password', String),
        Column('onboarded_orders', Boolean),
        Column('onboarded_listings', Boolean),
        Column('is_mfa', Boolean),
        Column('invalid_credentials', Boolean),
        Column('mfa_triggered_at', DateTime),
    )

def init_alias_account(meta, engine):
    Table(
        'aliasAccount', meta,
        Column('id', String, primary_key=True),
        Column('account_id', String),
        Column('email',  String),
        Column('password',  String),
        Column('onboarded_orders',  Boolean),
        Column('onboarded_listings',  Boolean),
        Column('is_mfa',  Boolean),
        Column('invalid_credentials',  Boolean),
        Column('mfa_triggered_at',  DateTime),
        Column('access_token',  String),
        Column('access_token_expiry',  Integer),
        Column('refresh_token',  String),
        Column('token_type',  String),
        Column('seller_score',  Integer)
    )

def init_alias_link(meta, engine):
    Table(
        'aliasAccountLink', meta,
        Column('id',                  String),
        Column('account_id',          String),
        Column('alias_account_id',    String),
        Column('email',               String),
        Column('status',              String),
        Column('account_type',        String)
    )

def init_blacklist_table(meta, engine):
    Table(
        'blacklist_tokens', meta,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('token', String, unique=True, nullable=False),
        Column('blacklisted_on', DateTime, nullable=False),
    )

def init_goat_seller_score_table(meta, engine):
    Table(
        'filghtclubSellerScore', meta,
        Column('id', String, primary_key=True),
        Column('account_id', String),
        Column('seller_flat_fee', Integer),
        Column('seller_transaction_fee', Float),
        Column('seller_score', Integer),
    )

def init_user_event_table(meta, engine):
    Table(
        'userEvent', meta,
        Column('id',                                 String, primary_key=True),
        Column('account_id',                         String),
        Column('event_type',                         String),
        Column('event_title',                        String),
        Column('event_text',                         String),
        Column('event_link',                         String),
        Column('event_time',                         String),
        Column('event_image',                        String),
    )

def init_ReportCardResult_table(meta, engine):
    Table(
        'reportCardResult', meta,
        Column('id',                                 String, primary_key=True),
        Column('account_id',                         String),
        Column('created_at', Date),
        Column('report_card_type', String),
        Column('profitable_sales_missed_count', Integer, default=0),
        Column('profit_missed_dollars', Float, default=0),
        Column('total_sales', Integer, default=0),
        Column('revenue_dollars', Float, default=0),
        Column('total_purchases', Integer, default=0),
        Column('purchase_cost_dollars', Float, default=0),
        Column('active_listings', Integer, default=0),
        Column('active_listings_value_dollars', Float, default=0),
    )


def init_monthly_proft_cache(meta, engine):
    Table(
        'userMonthlyProfit', meta,
        Column('id',                                 String, primary_key=True),
        Column('account_id',                         String),
        Column('created_at', Date),
        Column('sold_month', Date),
        Column('montly_profit', Float)
    )

def init_daily_proft_cache(meta, engine):
    Table(
        'userDailyProfit', meta,
        Column('id',                                 String, primary_key=True),
        Column('account_id',                         String),
        Column('created_at', Date),
        Column('sold_date', Date),
        Column('daily_profit', Float)
    )

def init_weekly_revenue(meta, engine):
    Table(
        'userWeeklyStats', meta,
        Column('id',                                 String, primary_key=True),
        Column('account_id',                         String),
        Column('created_at', Date),
        Column('sold_week', String),
        Column('weekly_revenue', Float),
        Column('weekly_profit', Float)
    )

def init_weekday_revenue_summary(meta, engine):
    Table(
        'userWeekdayRevenueSummary', meta,
        Column('id', String, primary_key=True),
        Column('account_id', String),
        Column('created_at', Date),
        Column('sold_weekday', Integer),
        Column('weekday_revenue', Float),
    )


def init_user_account_overview(meta, engine):
    Table(
        'userAccountOverview', meta,
        Column('id',                                        String, primary_key=True),
        Column('account_id',                                String, ),
        Column('created_at',                                DateTime),
        Column('historical_revenue',                        Integer),
        Column('number_of_sneakers',                        Integer),
        Column('current_inventory_secondary_market_value',  Float),
        Column('average_listing_price',                     Float),
        Column('historical_profit',                         Float),

    )

def init_all_tables(meta: MetaData, engine):
    # init_weekday_revenue_summary(meta, engine)
    # init_sale_order_table(meta, engine)
    init_user_account_overview(meta, engine)
    meta.create_all(engine)

def drop_all_tables(engine):
    SyncUserValueChain.__table__.drop(engine)
