import os
from logging import Logger
from types import FunctionType
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, Session as TSession
from sqlalchemy.orm import scoped_session
from sqlalchemy import MetaData

from suplexedlibrary.database.postgres.base import base
from suplexedlibrary.database.postgres import init_tables
from suplexedlibrary.model.DataSyncAccount import DataSyncAccount
from suplexedlibrary.model.AskGoat import AskGoat
from suplexedlibrary.model.AskStockX import AskStockX
from suplexedlibrary.model.ResellPricing import ResellPricing
from suplexedlibrary.model.GoatAccount import GoatAccount
from suplexedlibrary.model.InventoryItem import InventoryItem
from suplexedlibrary.model.PricingStrategy import PricingStrategy
from suplexedlibrary.model.SyncUserValueChain import SyncUserValueChain    


POSTGRES_HOST     = os.environ.get('POSTGRES_HOST') or 'emerald-island-postgres.cu5dqyiryars.us-east-2.rds.amazonaws.com'
POSTGRES_PORT     = os.environ.get('POSTGRES_PORT') or '5432'
POSTGRES_USERNAME = os.environ.get('POSTGRES_USERNAME') or 'emerald' 
POSTGRES_PASSWORD = os.environ.get('POSTGRES_PASSWORD') or 'Milo2021!'
POSTGRES_DB       = os.environ.get('POSTGRES_DB') or 'assist'

meta = None
Session = None
ScopedSession = None




def init_db():
    print(f'init db')
    global engine
    global Session
    global ScopedSession
    db_string = f"postgresql://{POSTGRES_USERNAME}:{POSTGRES_PASSWORD}@{POSTGRES_HOST}:{POSTGRES_PORT}/{POSTGRES_DB}"
    print(f'connecting to postgres database\n\tHOST={POSTGRES_HOST}\n\tPORT={POSTGRES_PORT}\n\tUSERNAME={POSTGRES_USERNAME}\n\tPASSWORD={POSTGRES_PASSWORD}\n\tDB={POSTGRES_DB}')
    engine = create_engine(db_string, pool_size=30, max_overflow=20)
    meta = MetaData(engine)
    base.metadata.create_all(engine)
    print('finished createting base metadata')
    Session = sessionmaker(engine)
    ScopedSession = scoped_session(Session)


def commit_transaction(db_session: TSession, logger: Logger = None, error_callback: FunctionType = None):
    try:
        db_session.commit()
    except Exception as e:
        logger.error('rolling back trasaction', exc_info=e) if logger else print('rolling back transaction')
        db_session.rollback()
        if error_callback:
            error_callback()
        raise Exception('Transaction Failed')


def get_session() -> TSession:
    return Session() if Session else None


def close_and_cleanup_db():
    print('closing all sessions...')
    Session.close_all()
    print('closing all scoped sessions...')
    ScopedSession.close_all()
    print('disposing of engine')
    engine.dispose()

def db_close_connections(db_session: TSession = None):
    Session.close_all()
    ScopedSession.close_all()

def get_scoped_session() -> TSession:
    return ScopedSession()

def get_new_session() -> TSession:
    return Session()

'''
def reset_tables():
    init_tables.drop_all_tables(engine)
    init_tables.init_all_tables(meta, engine)

def update_tables():
    init_tables.init_all_tables(meta, engine)
'''

