from pprint import pprint
from uuid import uuid4
from dateutil.parser import parse
from typing import List, Set
from datetime import datetime, timedelta
from requests import Session
from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor
import requests
from requests.api import get
from sqlalchemy import or_, func, and_
from sqlalchemy.orm import Session as TSession
from suplexedlibrary.constants.UserEventType import UserEventType

from suplexedlibrary.database.postgres.postgres import get_session, init_db, init_db
from suplexedlibrary.exceptions import FailedFetchingListings
from suplexedlibrary.inventory.InventoryManager import InventoryManager
from suplexedlibrary.api.GoatApi import GoatApi
from suplexedlibrary.model.GoatAccount import GoatAccount
from suplexedlibrary.constants.Marketplace import Marketplace
from suplexedlibrary.database.postgres.postgres import get_session

from suplexedlibrary.model.SaleOrder import SaleOrder
from suplexedlibrary.model.PurchaseOrder import PurchaseOrder
from suplexedlibrary.model.DataSyncAccount import DataSyncAccount
from suplexedlibrary.model.AskGoat import AskGoat
from suplexedlibrary.model.InventoryItem import InventoryItem
from suplexedlibrary.constants.Marketplace import Marketplace
from suplexedlibrary.discord import hooks
from suplexedlibrary.constants.ListingStatus import ListingStatus
from suplexedlibrary.constants.OrderStatusGoat import OrderStatusGoat
from suplexedlibrary.model.UserEvent import UserEvent

class InventoryManagerGoat(InventoryManager):
    def __init__(self, account: DataSyncAccount, goat_account: GoatAccount, session: Session, database: TSession = get_session()):
        super().__init__(account, database)
        self.goat_account = goat_account
        self.goat_api = GoatApi(goat_account, skip_login=False)
        self.listings = []
        self.goat_storage_listing = []
        self.purchase_orders = []
        self.sales = []

    def get_and_set_listings(self):
        try:
            self.listings = self.goat_api.get_listings()
        except FailedFetchingListings as e:
            print(f'caught listing fetch failure: {e}')
            self.is_listing_data_healthy = False
        except Exception as e:
            print(e)
            self.is_listing_data_healthy = False

    def get_and_set_goat_storage_listings(self):
        try:
            self.goat_storage_listing = self.goat_api.get_goat_storage_listings()  
        except FailedFetchingListings as e:
            print(f'caught listing fetch failure: {e}')
            self.is_listing_data_healthy = False
        except Exception as e:
            print(e)
            self.is_listing_data_healthy = False

    def get_and_set_purchase_orders(self):
        try:
            self.purchase_orders = self.goat_api.get_all_orders() 
        except Exception as e:
            print(e)

    def get_and_set_sale_orders(self):
        try:
            self.sales = self.goat_api.get_all_sell_orders()
        except Exception as e:
            print(e)

    def hydrate_listings(self):
        hydration_tasks = [
            self.get_and_set_listings,
            self.get_and_set_goat_storage_listings
        ]
        with ThreadPoolExecutor(1) as executor:
            [executor.submit(task) for task in hydration_tasks]

    def hydrate_orders(self):
        hydration_tasks = [
            self.get_and_set_purchase_orders,
            self.get_and_set_sale_orders
        ]
        with ThreadPoolExecutor(1) as executor:
            [executor.submit(task) for task in hydration_tasks]        

    def hydrate_account(self):     
        hydration_tasks = [
            self.get_and_set_listings,
            self.get_and_set_goat_storage_listings,
            self.get_and_set_purchase_orders,
            self.get_and_set_sale_orders
        ]
        with ThreadPoolExecutor(1) as executor:
            [executor.submit(task) for task in hydration_tasks]

    def process_listings(self):
        print('listing processor')

        total_listings = self.goat_storage_listing + self.listings
        total_listings = list(filter(lambda x : x['saleStatus'] == 'active', total_listings))

        print(f'{len(total_listings)} to process')
        for listing in total_listings:
            self.create_inventory_and_ask_from_listing_in_db(
                shoe_name=listing['productTemplate']['name'],
                shoe_size=listing['size'],
                shoe_sku=listing['productTemplate']['sku'].replace(' ', '-'),
                ask_amount=listing['priceCents'] / 100,
                exchange='goat',
                exchange_ask_id=listing['id'],
                image_url=listing['mainPictureUrl'],
                marketplace='goat',
                created_at=listing['createdAt'],
                is_consigned=listing['consigned'] if 'consigned' in listing else False,
                status='active' if listing['saleStatus'] == 'active' else 'inactive',
                product_type=self.parse_goat_product_type(listing['productTemplate']['productType']),
                goat_product_url_key=listing['shortUrl']
            )
            

    def parse_goat_product_type(self, product_type: str):
        if product_type == 'sneakers':
            return 'sneaker'

    def process_purchase_orders(self, pages=2):

        for order in self.purchase_orders:

            purchase_order_id = str(order['id'])
            matched_purchase_order = self.db.query(PurchaseOrder).filter(
                PurchaseOrder.exchange_ask_id == purchase_order_id,
                PurchaseOrder.account_id == self.account.id
            ).first()
            if not matched_purchase_order:
                self.create_purchase_order_in_db(
                    account_id=self.account.id,
                    shoe_sku=order['product']['productTemplate']['sku'],
                    shoe_name=order['product']['productTemplate']['name'],
                    shoe_size=order['product']['size'],
                    inventory_id=None,
                    exchange_ask_id=order['id'],
                    purchase_date=order['purchasedAt'],
                    purchase_fee=GoatApi.get_goat_amount_dollars(order['taxCents']) if 'taxCents' in order else 0,
                    purchase_amount=GoatApi.get_goat_amount_dollars(order['priceCents']),
                    purchase_shipping=GoatApi.get_goat_amount_dollars(1000),
                    purchase_tax=GoatApi.get_goat_amount_dollars(order['taxCents']) if 'taxCents' in order else 0,
                    purchase_discount=0,
                    marketplace=Marketplace.GOAT,
                    image_url=order['product']['mainPictureUrl'],
                    tracking_url=order['trackingToBuyerCodeUrl'] if 'trackingToBuyerCodeUrl' in order else None,
                    tracking_delivery_date=None,
                    tracking_number=order['trackingToBuyerCode'] if 'trackingToBuyerCode' in order else None,
                    tracking_status=None,
                    order_status=order['buyerTitle'],
                    status_code=order['status'],
                    order_number=order['number'],
                    purchase_method=None,
                    authentication_fee=0
                )
            else:
                purchase_order = self.update_purchase_order_in_db(
                    matched_purchase_order,
                    order_status=order['sellerTitle'],
                    tracking_url=order['trackingToGoatCodeUrl'] if 'trackingToGoatCodeUrl' in order else None,
                    tracking_delivery_date=None,
                    tracking_number=order['trackingToGoatCode'] if 'trackingToGoatCode' in order else None,
                    tracking_status=None,
                    status_code=order['status'],
                    order_number=order['number'],
                    purchase_amount=GoatApi.get_goat_amount_dollars(order['priceCents']),
                    purchase_fee=GoatApi.get_goat_amount_dollars(order['taxCents']) if 'taxCents' in order else 0,
                    purchase_tax=GoatApi.get_goat_amount_dollars(order['taxCents']) if 'taxCents' in order else 0,
                    purchase_shipping=GoatApi.get_goat_amount_dollars(1000),                    
                )


    def process_sale_orders(self, pages=2):
        
        for sale in self.sales:
            if 'canceled' in sale['status'] or 'fraud' in sale['status']:
                print('this sale was canceled, continuing')
                continue
            
            sale_order_id = str(sale['product']['id'])
            matched_sale_order = self.db.query(SaleOrder).filter(
                SaleOrder.exchange_ask_id == sale_order_id,
                SaleOrder.account_id == self.account.id,
                and_(
                    SaleOrder.order_status != 'Fraud Prevented'
                )
            ).first()

            if not matched_sale_order: # If the sale is new, create new objects for it 
                self.create_sale_event({
                    'account_id': self.account.id,
                    'shoe_sku': sale['product']['productTemplate']['sku'],
                    'shoe_name': sale['product']['productTemplate']['name'],
                    'shoe_size': sale['product']['size'],
                    'exchange_ask_id': sale_order_id,
                    'sold_date': sale['purchasedAt'],
                    'sold_amount': GoatApi.get_goat_amount_dollars(sale['priceCents']),
                    'sold_payout': GoatApi.get_goat_amount_dollars(sale['sellerAmountMadeCents']),
                    'sold_transaction_fee': GoatApi.get_goat_amount_dollars(500),
                    'sold_payment_processing_fee': GoatApi.get_goat_amount_dollars(sale['priceCents'] - sale['sellerAmountMadeCents'] - 500),
                    'sold_shipping': 0,
                    'sold_discount': 0,
                    'sold_tax': GoatApi.get_goat_amount_dollars(sale['taxCents']) if 'taxCents' in sale else 0,
                    'marketplace': Marketplace.GOAT.value,
                    'image_url': sale['product']['mainPictureUrl'],
                    'order_status': sale['sellerTitle'],
                    'tracking_url': sale['trackingToGoatCodeUrl'] if 'trackingToGoatCodeUrl' in sale else None,
                    'tracking_delivery_date': None,
                    'tracking_number': sale['trackingToGoatCode'] if 'trackingToGoatCode' in sale else None,
                    'tracking_status': None,
                    'status_code': sale['status'],
                    'order_number': sale['number'],
                    'payout_destination': '',
                    'listing_creation_date': sale['product']['createdAt'],
                    'shipping_label_url': sale['shippingLabelToGoat'] if 'shippingLabelToGoat' in sale else None,
                    'is_sale_confirmed': sale['status'].lower() != OrderStatusGoat.SellerProgress.NEEDS_CONFIRMATION.lower() and sale['status'].lower() != OrderStatusGoat.SellerProgress.SOLD.lower()
                })

            else: # If the sale exists, update objects for it
                self.update_sale_event({
                    'marketplace': 'goat',
                    'account_id': self.account.id,
                    'sale_order_id': sale_order_id,
                    'order_status': sale['sellerTitle'],
                    'tracking_url': sale['trackingToGoatCodeUrl'] if 'trackingToGoatCodeUrl' in sale else None,
                    'tracking_delivery_date': None,
                    'tracking_number': sale['trackingToGoatCode'] if 'trackingToGoatCode' in sale else None,
                    'tracking_status': None,
                    'status_code': sale['status'],
                    'order_number': sale['number'],
                    'sold_amount': GoatApi.get_goat_amount_dollars(sale['priceCents']),
                    'sold_payout': GoatApi.get_goat_amount_dollars(sale['sellerAmountMadeCents']),
                    'sold_transaction_fee': GoatApi.get_goat_amount_dollars(500),
                    'sold_payment_processing_fee': GoatApi.get_goat_amount_dollars(sale['priceCents'] - sale['sellerAmountMadeCents'] - 500),
                    'sold_shipping': 0,
                    'sold_tax': GoatApi.get_goat_amount_dollars(sale['taxCents']) if 'taxCents' in sale else 0,
                    'shipping_label_url': sale['shippingLabelToGoat'] if 'shippingLabelToGoat' in sale else None,
                    'is_sale_confirmed': sale['status'].lower() != OrderStatusGoat.SellerProgress.NEEDS_CONFIRMATION.lower() and sale['status'].lower() != OrderStatusGoat.SellerProgress.SOLD.lower()                              
                })

    @staticmethod
    def process_missing_listings_static(
        listings: List[AskGoat], 
        account_id: str, 
        goat_api: GoatApi, 
        created_listings: Set[str]
    ):
        print(f'processing missing listings for {account_id}')
        db_session = get_session()
        all_asks = db_session.query(AskGoat).filter(
            AskGoat.account_id == account_id
        ).all()
        ask_set = {}
        exchange_ask_set = {}

        for ask in all_asks:
            ask_set[ask.exchange_ask_id] = ask
        
        original_size = len(ask_set)

        print(f'\t{len(listings)} on assist goat account in RDS')

        for ask in listings:
            ask_set.pop(ask['id'], None)
            exchange_ask_set[ask['id']] = ask

        for stale_ask in ask_set:
            if ask_set[stale_ask].status == ListingStatus.PENDING:
                print(f'\tthis ask is still pending do not delete: {stale_ask}')
                continue

            if stale_ask in created_listings:
                print(f'\tthis ask was just created, do not delete: {stale_ask}')
                continue
            
            inventory_item = db_session.query(InventoryItem).filter(InventoryItem.id == ask_set[stale_ask].inventory_item_id).first()

            print(f'\tremoving stale ask:{stale_ask}, [style:{ask_set[stale_ask].style_id} | size:{ask_set[stale_ask].size}]')
            user_event = UserEvent(
                id                                 = str(uuid4()),
                account_id                         = account_id,
                inventory_id                       = inventory_item.id,       
                sale_order_id                      = None,
                purchase_order_id                  = None,               
                event_type                         = UserEventType.LISTING_REMOVE_OFF_ASSIST,
                event_title                        = f'Listing removed off of sync platform for {inventory_item.shoeName} size {inventory_item.shoeSize} on Goat',
                event_text                         = f'Listing removed off of sync platform for {inventory_item.shoeName} size {inventory_item.shoeSize} on Goat',
                event_link                         = None,
                event_time                         = datetime.now(),
                event_image                        = inventory_item.image_url                
            )
            db_session.add(user_event)
            db_session.delete(ask_set[stale_ask])
            db_session.commit()

if __name__ == '__main__':
    init_db()
    db_session = get_session()

    data_sync_account = db_session.query(DataSyncAccount).filter(DataSyncAccount.email == 'daniel.revie1@gmail.com').first()
    goat_account = db_session.query(GoatAccount).filter(GoatAccount.account_id == data_sync_account.id, GoatAccount.email == 'daniel.revie1@gmail.com').first()

    inventory_manager = InventoryManagerGoat(account=data_sync_account, goat_account=goat_account, session=requests.session(), database=db_session)

    inventory_manager.get_and_set_sale_orders()
    inventory_manager.process_sale_orders()

    print('main')