
from pprint import pprint
from requests import Session
from suplexedlibrary.inventory.InventoryManager import InventoryManager
from suplexedlibrary.model.StadiumGoodsAccount import StadiumGoodsAccount
from suplexedlibrary.model.DataSyncAccount import DataSyncAccount
from suplexedlibrary.api.StadiumGoodsApi import StadiumGoodsApi
from suplexedlibrary.model.SaleOrder import SaleOrder
from suplexedlibrary.proxies.provider import get_luminati_sync_static_residential
from suplexedlibrary.database.postgres.postgres import get_session
from suplexedlibrary.constants.Marketplace import Marketplace

class InventoryManagerStadiumGoods(InventoryManager):

    def __init__(self, account: DataSyncAccount, stadium_goods_account: StadiumGoodsAccount, session: Session):
        super().__init__(account)
        self.stadium_goods_account = stadium_goods_account
        self.sess = session
        self.stadium_goods_api = StadiumGoodsApi()
        self.stadium_goods_api.hydrate_login(self.sess, stadium_goods_account)

    def process_listings():
        pass

    def process_purchase_orders():
        pass

    def process_sale_orders(self):
        sales = self.stadium_goods_api.get_sold_orders()
        for sale in sales:
            matched_sale_order = get_session().query(SaleOrder).filter(SaleOrder.exchange_ask_id == sale['exchange_id']).first()
            
            if matched_sale_order:
                print(matched_sale_order.marketplace)
                continue

            self.create_sale_order_in_db(
                    account_id=self.account.id,
                    shoe_sku=sale['style_id'],
                    shoe_name=sale['title'],
                    shoe_size=sale['size'],
                    inventory_id=None,
                    exchange_ask_id=sale['exchange_id'],
                    sold_date=sale['sold_on'],
                    sold_amount=sale['sold_price'],
                    sold_payout=sale['sold_price'] - sale['sold_price'] * .2,
                    sold_transaction_fee=sale['sold_price'] * .2,
                    sold_payment_processing_fee=None,
                    sold_shipping=0,
                    sold_discount=0,
                    marketplace=Marketplace.STADIUMGOODS,
                    image_url=sale['image_url'],
                    order_status=sale['status'],
                    tracking_url=None,
                    tracking_delivery_date=None,
                    tracking_number=None,
                    tracking_status=None,
                    status_code=sale['status'],
                    order_number=None,
                    payout_destination=None
            )

if __name__ == '__main__':
    account = get_session().query(DataSyncAccount).filter(DataSyncAccount.email == 'dan').first()
    stadium_goods_account = account.stadiumGoodsAccount[0]
    manager = InventoryManagerStadiumGoods(account, stadium_goods_account, get_luminati_sync_static_residential())
    manager.process_sale_orders()