import argparse
import concurrent
from concurrent.futures import ThreadPoolExecutor
from concurrent.futures import Future 
from requests import session, Session
from suplexedlibrary.database.postgres.postgres import get_session
from suplexedlibrary.model.DataSyncAccount import DataSyncAccount, StockxAccount
from suplexedlibrary.api.StockxApi import StockXApi
from suplexedlibrary.api.StockxMobileApi import StockxMobileApi
from suplexedlibrary.proxies import provider
from suplexedlibrary.discord import hooks
from suplexedlibrary.luminati.allocate_proxies import allocate_new_stockx_proxy

healthy_count = 0
newly_generated_count = 0
failed_count = 0

def is_stockx_login_healthy(stockx_account: StockxAccount, session: Session) -> bool:
    global healthy_count
    try:
        api = StockXApi(stockx_account).test_api_login()
        healthy_count += 1
        return True
    except Exception as e:
        print(e)
        return False


def process_login(account: DataSyncAccount):
    global failed_count
    global newly_generated_count
    
    stockx_account: StockxAccount = account.stockxAccount[0]
    sess = provider.get_sync_data_center_proxy()
    sess.headers.clear()
    if is_stockx_login_healthy(stockx_account, sess):
        print('User session is already healthy')
        # hooks.write_to_genreate_stockx_login_channel(account.email, f'login is still healthy')
        return

    
    attempt_count = 1
    while attempt_count < 3:
        try:
            print(stockx_account.email)


            sess.headers.clear()
            sess.cookies.clear_session_cookies()

            api = StockXApi(stockx_account, skip_login=True)
            api.new_stockx_login()
            newly_generated_count += 1
            break
        except Exception as e:
            print(e)
            sess = provider.get_sync_data_center_proxy()
            attempt_count += 1

    if attempt_count >= 3:
        failed += 1
    
    # hooks.write_to_genreate_stockx_login_channel(account.email, message)


def generate_logins(account_level: str):

    account_cursor = get_session().query(DataSyncAccount).filter(DataSyncAccount.stockxAccount != None, DataSyncAccount.email != 'DOITBIGTICKETS@GMAIL.COM').all()
    with ThreadPoolExecutor(1) as executor:
        for future in concurrent.futures.as_completed([executor.submit(process_login, account) for account in account_cursor]):
            print(future.result())

    # hooks.write_to_genreate_stockx_login_channel(f'\n\n{healthy_count} healthy logins remaining\n{newly_generated_count} logins created\n{failed_count} login attempts failed\n\n')

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='inventory processor arguments!')
    parser.add_argument("--accountLevel", default='beta')
    args = parser.parse_args()
    generate_logins(account_level=args.accountLevel)