import argparse
from requests import session, Session
from pprint import pprint

from suplexedlibrary.database.postgres.postgres import get_session
from suplexedlibrary.proxies import provider
from suplexedlibrary.model.GoatAccount import GoatAccount
from suplexedlibrary.model.DataSyncAccount import DataSyncAccount
from suplexedlibrary.api.GoatApi import GoatApi
from suplexedlibrary.discord import hooks

healthy_count = 0
newly_generated_count = 0
failed_count = 0


def is_login_healthy(goat_account: GoatAccount) -> bool:
    global healthy_count
    try:
        api = GoatApi(goat_account, skip_login=False).test_goat_login()
        healthy_count += 1
        return True
    except:
        return False


def generate_goat_login(account: DataSyncAccount):
    db_session = get_session()
    if not account.goatAccount:
        print(f'no goat login for {account.email}')
        return

    goat_account: GoatAccount = account.goatAccount[0]

    try:
        if goat_account.invalid_credentials:
            print(f'skipping {goat_account.email} because of invalid credentials')
            return

        print(f'testing health for {goat_account.email}')

        if is_login_healthy(goat_account):
            print('\tUser session is already healthy')
            return
        else:
            print(f'\tLogin session is unhealthy')

        print(f'\tGenerating new login for {goat_account.email}')
        api = GoatApi(account, skip_login=True)
        login_details = api.login(goat_account.email, goat_account.password)
        if not login_details:
            raise Exception('No login details were returned')

        if 'incorrectPassword' in login_details:
            goat_account.invalid_credentials = True
            db_session.commit()
            return

        pprint(login_details)
        goat_account.authorization = login_details['authorization']
        goat_account.x_emb_st = login_details['x_emb_st']
        goat_account.cookie = login_details['cookie']
        goat_account.x_emb_id = login_details['x_emb_id']
        goat_account.user_agent = login_details['user_agent']
        goat_account.user_id = login_details['user_id']
        goat_account.x_px_authorization = login_details['x_px_authorization']
        goat_account.x_px_original_token = login_details['x_px_original_token']
        db_session.commit()
    except Exception as e:
        print(e)
    finally:
        print('\n\n')


def generate_logins(account_level: str):
    global newly_generated_count
    global failed_count

    for data_sync_account in get_session().query(DataSyncAccount).filter(
        DataSyncAccount.account_level != None,
        DataSyncAccount.goatAccount != None,
    ).all():
        generate_goat_login(data_sync_account)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='inventory processor arguments!')
    parser.add_argument("--accountLevel", default='beta')
    args = parser.parse_args()
    generate_logins(account_level=args.accountLevel)
    # hooks.write_to_genreate_goat_login_channel(f'\n\n{healthy_count} healthy logins remaining\n{newly_generated_count} logins created\n{failed_count} login attempts failed\n\n')