import flask_bcrypt
import datetime
import jwt

from sqlalchemy import Column, String, DateTime, Integer
from .StockxAccount import StockxAccount
from .GoatAccount import GoatAccount
from .AskGoat import AskGoat
from .AskStockX import AskStockX
from .AskStadiumGoods import AskStadiumGoods
from .AskFlightClub import AskFlightClub
from .StadiumGoodsAccount import StadiumGoodsAccount
from .FlightClubAccount import FlightClubAccount
from .StockxSellerScore import StockxSellerScore
from .GoatSellerScore import GoatSellerScore
from .BlacklistToken import BlacklistToken
from suplexedlibrary.constants.AccountLevel import AccountLevel
from sqlalchemy.orm import relationship

from suplexedlibrary.database.postgres.base import base

key = 'my_precious_secret_key'
data_api_key = 'suplexed-69-05-1969-data-api-key'

class DataSyncAccount(base):
    is_authenticated: bool

    __tablename__ = 'dataSyncAccount'
    id                                     = Column(String, primary_key=True)
    first_name                             = Column(String)
    last_name                              = Column(String)
    date_of_birth                          = Column(String)
    email                                  = Column(String)
    registered_on                          = Column(DateTime)
    password_no_hash                       = Column(String)
    password_hash                          = Column(String)
    confirmed_email                        = Column(Integer)
    stripe_id                              = Column(String)
    stripe_subcsription_id                 = Column(String)
    stripe_subscription_product            = Column(String)
    stripe_subscription_status             = Column(String)
    discord_id                             = Column(String)
    orders_last_updated_time               = Column(DateTime)
    listings_last_updated_time             = Column(DateTime)
    shoe_size                              = Column(String)
    phone_number                           = Column(String)
    zip_code                               = Column(String)
    legacy_suplexed_id                     = Column(String)
    legacy_onesignal_id                    = Column(String)
    suplexed_api_auth_token                = Column(String)
    assist_api_stripe_subcsription_id      = Column(String)
    assist_api_stripe_subscription_product = Column(String)
    assist_api_stripe_subscription_status  = Column(String)
    goatAccount                            = relationship(GoatAccount)
    stockxAccount                          = relationship(StockxAccount)
    stadiumGoodsAccount                    = relationship(StadiumGoodsAccount)
    flightClubAccount                      = relationship(FlightClubAccount)
    goatAsk                                = relationship(AskGoat, backref='dataSyncAccount')
    stockxAsks                             = relationship(AskStockX, backref='dataSyncAccount')
    stadiumGoodsAsks                       = relationship(AskStadiumGoods, backref='dataSyncAccount')
    flightClubAsks                         = relationship(AskFlightClub, backref='dataSyncAccount')
    pricingStrategy                        = relationship('PricingStrategy')
    inventory                              = relationship('InventoryItem')
    syncUserValueChain                     = relationship('SyncUserValueChain')
    stockxSellerScore                      = relationship('StockxSellerScore', back_populates='dataSyncAccount', uselist=False)
    goatSellerScore                        = relationship('GoatSellerScore', back_populates='dataSyncAccount', uselist=False)
    flightclubSellerScore                  = relationship('FlightClubSellerScore', back_populates='dataSyncAccount', uselist=False)


    
    @property
    def password(self):
        raise AttributeError('password: write-only field')

    @password.setter
    def password(self, password: str):
        self.password_hash = flask_bcrypt.generate_password_hash(password).decode('utf-8')

    

    def check_password(self, password: str):
        return flask_bcrypt.check_password_hash(self.password_hash, password)

    @staticmethod
    def encode_suplexed_api_auth_token(user_id: str) -> str:
        try: 
            payload = {
                'exp': datetime.datetime.utcnow() + datetime.timedelta(days=365 * 20),
                'iat': datetime.datetime.utcnow(),
                'sub': user_id                
            }
            return jwt.encode(
                payload,
                data_api_key,
                algorithm='HS256'
            )            
        except Exception as e:
            print(e)
            return e

    @staticmethod
    def decode_suplexed_api_auth_token(auth_token: str):
        try:
            return jwt.decode(auth_token, data_api_key)['sub']
        except jwt.ExpiredSignatureError:
            raise Exception('Signature expired. Please log in again.')
        except jwt.InvalidTokenError:
            raise Exception('Invalid token. Please log in again.')

    @staticmethod
    def encode_auth_token(user_id: str) -> str:
        """
        Generates the Auth Token
        :return: string
        """
        try:
            payload = {
                'exp': datetime.datetime.utcnow() + datetime.timedelta(days=365 * 2),
                'iat': datetime.datetime.utcnow(),
                'sub': user_id
            }
            return jwt.encode(
                payload,
                key,
                algorithm='HS256'
            )
        except Exception as e:
            return e

    @staticmethod
    def decode_auth_token(auth_token, db_session):
        """
        Decodes the auth token
        :param auth_token:
        :return: integer|string
        """
        try:
            payload = jwt.decode(auth_token, key)
            is_blacklisted_token = BlacklistToken.check_blacklist(auth_token, db_session)
            if is_blacklisted_token:
                raise Exception('Please log in again.')
            else:
                return payload['sub']

        except jwt.ExpiredSignatureError:
            raise Exception('Signature expired. Please log in again.')
        except jwt.InvalidTokenError:
            raise Exception('Invalid token. Please log in again.')

    def is_active(self):
        """True, as all users are active."""
        return True

    def get_id(self):
        """Return the email address to satisfy Flask-Login's requirements."""
        return self.email

    def is_authenticated(self):
        """Return True if the user is authenticated."""
        return self.authenticated

    def is_anonymous(self):
        """False, as anonymous users aren't supported."""
        return False