from suplexedlibrary.database.postgres.base import base
from datetime import datetime

from sqlalchemy import Column, String, ForeignKey, Constraint, Float, Boolean, TIMESTAMP, func, DateTime
from sqlalchemy.orm import relationship
from .ResellPricing import ResellPricing
from .AskStockX import AskStockX
from .AskGoat import AskGoat
from .PricingStrategy import PricingStrategy
from .PurchaseOrder import PurchaseOrder
from .SaleOrder import SaleOrder
from .AskStadiumGoods import AskStadiumGoods
from .InventorySync import InventorySync
from .AskFlightClub import AskFlightClub

class InventoryItem(base):
    __tablename__ = 'inventoryItem'

    id                      = Column(String, primary_key=True)
    account_id              = Column(String, ForeignKey('dataSyncAccount.id'))
    shoeName                = Column(String)
    shoeSize                = Column(String)
    shoeSku                 = Column(String)
    image_url               = Column(String)
    synced                  = Column(Boolean)
    product_type            = Column(String)
    is_spec                 = Column(Boolean)

    stockx_product_url_key  = Column(String)
    goat_product_url_key    = Column(String)

    created_at              = Column(DateTime, default=datetime.now)
    updated_at              = Column(DateTime, default=datetime.now, onupdate=datetime.now)
    last_updated_time       = Column(TIMESTAMP, server_default=func.now(), onupdate=func.current_timestamp(), default=func.current_timestamp())

    inventorySync           = relationship(InventorySync, back_populates='inventoryItem', uselist=False)
    pricingStrategy         = relationship(PricingStrategy, back_populates='inventoryItem', uselist=False)
    askGoat                 = relationship(AskGoat, back_populates='inventoryItem', uselist=False)
    askStockX               = relationship(AskStockX, back_populates='inventoryItem', uselist=False)
    askStadiumGoods         = relationship(AskStadiumGoods, back_populates='inventoryItem', uselist=False)
    askFlightClub           = relationship(AskFlightClub, back_populates='inventoryItem', uselist=False)
    resellPricing           = relationship(ResellPricing, back_populates='inventoryItem', uselist=False)
    purchaseOrder           = relationship(PurchaseOrder, back_populates='inventoryItem', uselist=False)
    saleOrder               = relationship(SaleOrder, back_populates='inventoryItem', uselist=False)