from pprint import pprint
import random
import time
import traceback
from typing import List

from suplexedlibrary.model.AskStockX import AskStockX
from suplexedlibrary.database.postgres.postgres import get_session, init_db
from suplexedlibrary.model.StockxAccount import StockxAccount
from suplexedlibrary.model.InventoryItem import InventoryItem
from suplexedlibrary.api.StockxApi import StockXApi
from suplexedlibrary.exceptions import DeleteListingNotFoundException, DeleteListingFailedException, Scraping403Exception

init_db()
db_session = get_session()


def process_stockx_deletes(
    dont_delete_set: set, 
    delete_list: List[dict],
    stockx_api: StockXApi
):
    for item in delete_list:
        chain_id = item['chainId']
        if chain_id in dont_delete_set:
            continue
        
        try:
            stockx_api.remove_ask(chain_id)
            db_session.query(AskStockX).filter(AskStockX.exchange_ask_id == chain_id).delete()
        except DeleteListingNotFoundException as e:
            print('already deleted on api, removing from DB')
            db_session.query(AskStockX).filter(AskStockX.exchange_ask_id == chain_id).delete()
        except Exception as e:
            print(e)
        finally:
            time.sleep(random.randint(3, 9))


def delete_all_stockx():
    print('deleting account')
    account = db_session.query(StockxAccount).filter(StockxAccount.email == 'sneakervaultecho@gmail.com').first()
    api = StockXApi(account, skip_login=False)


    current_selling_gql = api.delete_all_asks(sell_type='CURRENT')
    #print(f'deleting {len(current_selling_gql)} stockx inventory items')
#
    #process_stockx_deletes(set(), current_selling_gql, api)
           
def delete_all_non_synced_stockx():
    account = db_session.query(StockxAccount).filter(StockxAccount.email == 'sneakervaultecho@gmail.com').first()
    api = StockXApi(account)
    synced_inventory = db_session.query(InventoryItem).filter(
        InventoryItem.account_id == account.id, 
        InventoryItem.synced == True,
        InventoryItem.askStockX != None,
        InventoryItem.is_spec == False
    ).all()

    synced_inventory_set = set()
    print(f'found {len(synced_inventory)} synced inventory')
    
    for item in synced_inventory:
        synced_inventory_set.add(item.askStockX.exchange_ask_id)
    pprint(synced_inventory_set)
    # current_selling = api.get_all_sales_(sale_type='current')

    current_selling_gql = api.get_sales_history_gql(sell_type='CURRENT')

    process_stockx_deletes(synced_inventory_set, current_selling_gql, api)


def delete_all_non_specs():
    account = db_session.query(StockxAccount).filter(StockxAccount.email == 'sneakervaultbuyerbeta@gmail.com').first()
    api = StockXApi(account)

    account_id = account.account_id

    non_synced_inventory = db_session.query(InventoryItem).filter(
        InventoryItem.account_id == account_id, 
        InventoryItem.synced == False,
        InventoryItem.askStockX != None,
        InventoryItem.is_spec == False
    ).all()

    for item in non_synced_inventory:

        print(f'{item.shoeSku} -- {item.shoeSize}')

        try:
            stockx_ask: AskStockX = item.askStockX
            api.remove_ask(stockx_ask.exchange_ask_id)
            db_session.delete(stockx_ask)
            db_session.commit()
        except DeleteListingNotFoundException as e:
            db_session.delete(stockx_ask)
            db_session.commit()
        except DeleteListingFailedException as e:
            print(e)
        finally:
            time.sleep(random.randint(1, 3))


if __name__ == '__main__':
    while True:
        try:
            delete_all_stockx()
        except Scraping403Exception as e:
            sleep_time = random.randint(60 * .5, 60 * 2)
            print(f'[{e}] failed, sleeping for {sleep_time / 60} minutes')
            time.sleep(sleep_time)
            continue
        except Exception as e:
            traceback.print_exc()
            continue
            