
from suplexedlibrary.model.PricingStrategy import PricingStrategy
from suplexedlibrary.model.InventoryItem import InventoryItem
from suplexedlibrary.model.AskStockX import AskStockX
from suplexedlibrary.model.AskGoat import AskGoat
from suplexedlibrary.model.AskFlightClub import AskFlightClub
from suplexedlibrary.model.AskStadiumGoods import AskStadiumGoods
from suplexedlibrary.database.postgres.postgres import TSession
from suplexedlibrary.constants.ListingStatus import ListingStatus
from suplexedlibrary.constants.PricerStrategy import PricerStrategy
from sqlalchemy import or_, and_

class AbstractSuplexedAccountLevel:
    # Maximum number of active listings a user can have
    max_active_listings: int

    # Maximum number of active synced listings a user can have
    max_active_synced_listings: int

    # Maximum number of active pricing strategies a user can have
    max_active_pricing_strategies: int

    db_session: TSession

    account_id: str


    def __init__(
        self,
        max_active_listings: int,
        max_active_synced_listings: int,
        max_active_pricing_strategies: int,
        account_id: str,
        db_session: TSession,
        is_hydrate_all: bool = True
    ) -> None:
        self.max_active_listings = max_active_listings
        self.max_active_synced_listings = max_active_synced_listings
        self.max_active_pricing_strategies = max_active_pricing_strategies
        self.account_id = account_id
        self.db_session = db_session

        if is_hydrate_all:
            self.hydrate_cursors()


    def hydrate_cursors(self):
        inventory_items = set()

        self.active_flightclub_listings = self.db_session.query(AskFlightClub).filter(
            AskFlightClub.account_id == self.account_id, AskFlightClub.status == ListingStatus.ACTIVE
        ).all()
        
        self.active_goat_listings = self.db_session.query(AskGoat).filter(
            AskGoat.account_id == self.account_id, AskGoat.status == ListingStatus.ACTIVE
        ).all() 

        self.active_stockx_listings = self.db_session.query(AskStockX).filter(
            AskStockX.account_id == self.account_id, AskStockX.status == ListingStatus.ACTIVE
        ).all()

        self.active_stadiumgoods_listings = self.db_session.query(AskStadiumGoods).filter(
            AskStadiumGoods.account_id == self.account_id, AskStadiumGoods.status == ListingStatus.ACTIVE
        ).all()     

        
        for item in self.active_flightclub_listings:
            inventory_items.add(item.inventory_item_id)

        for item in self.active_goat_listings:
            inventory_items.add(item.inventory_item_id)

        for item in self.active_stockx_listings:
            inventory_items.add(item.inventory_item_id)

        for item in self.active_stadiumgoods_listings:
            inventory_items.add(item.inventory_item_id)    

        self.inventory_items = []
        for item in inventory_items:
            self.inventory_items.append(self.db_session.query(InventoryItem).filter(InventoryItem.account_id == self.account_id, InventoryItem.id == item).first())


    def get_next_listing_count(self):
        return len(self.active_flightclub_listings) + len(self.active_goat_listings) + len(self.active_stockx_listings) + len(self.active_stadiumgoods_listings) + 1

    def get_next_active_synced_count(self):
        count = 0
        for item in self.inventory_items:
            if item.synced == True:
                count += 1
    
        return count

    def get_next_active_pricing_strategy_count(self):
        floor_pricing_count = 0

        for inventory_item in self.inventory_items:
            if inventory_item.pricingStrategy.strategy_type == PricerStrategy.FLOOR:
                floor_pricing_count += 1

        return floor_pricing_count



    def is_max_active_listings_reached(self):
        return self.max_active_listings < self.get_next_listing_count()


    def is_max_active_synced_listings_reached(self):
        return self.max_active_synced_listings < self.get_next_active_synced_count()


    def is_max_max_active_pricing_strategies_reached(self):
        return self.max_active_pricing_strategies < self.get_next_active_pricing_strategy_count()