from sqlalchemy.sql.operators import is_
from suplexedlibrary.model.DataSyncAccount import DataSyncAccount
from suplexedlibrary.paywall.AbstractSuplexedAccountLevel import AbstractSuplexedAccountLevel
from suplexedlibrary.paywall.SuplexedAccountLevel_One import SuplexedAccountLevel_One
from suplexedlibrary.paywall.SuplexedAccountLevel_Two import SuplexedAccountLevel_Two
from suplexedlibrary.paywall.SuplexedAccountLevel_Three import SuplexedAccountLevel_Three
from suplexedlibrary.database.postgres.postgres import TSession, get_session, init_db

class PaywallUtil:
    STANDARD_MONTHLY_PRICE_ID = 'price_1IPFJZG6XLeGzfOcHaGWdXyU'
    STANDARD_YEARLY_PRICE_ID = ''
    PREMIUM_MONTHLY_PRICE_ID = 'price_1IPFHYG6XLeGzfOcq5XFzWjr'
    PREMIUM_YEARLY_PRICE_ID = ''

    @staticmethod
    def get_suplexed_account_level(account_id: str, db_session: TSession) -> AbstractSuplexedAccountLevel:
        account = db_session.query(DataSyncAccount).filter(DataSyncAccount.id == account_id).first()

        stripe_subscription_product = account.stripe_subscription_product

        if stripe_subscription_product == PaywallUtil.STANDARD_MONTHLY_PRICE_ID:
            print(f'found {account_id} is level_one with product_id:{stripe_subscription_product}')
            return SuplexedAccountLevel_One(account_id=account_id, db_session=db_session, is_hydrate_all=True)

        if stripe_subscription_product == PaywallUtil.PREMIUM_MONTHLY_PRICE_ID:
            print(f'found {account_id} is level_two with product_id:{stripe_subscription_product}')
            return SuplexedAccountLevel_Two(account_id=account_id, db_session=db_session, is_hydrate_all=True)            

        print(f'no account level could be found for account_id={account_id}, product={stripe_subscription_product}')
        raise Exception(f'Could not find account level for price_id={stripe_subscription_product}')

    @staticmethod
    def get_suplexed_account_level_unhydrated(account_id: str, db_session: TSession) -> AbstractSuplexedAccountLevel:
        account = db_session.query(DataSyncAccount).filter(DataSyncAccount.id == account_id).first()

        stripe_subscription_product = account.stripe_subscription_product

        if stripe_subscription_product == PaywallUtil.STANDARD_MONTHLY_PRICE_ID:
            print(f'found {account_id} is level_one with product_id:{stripe_subscription_product}')
            return SuplexedAccountLevel_One(account_id=account_id, db_session=db_session, is_hydrate_all=False)

        if stripe_subscription_product == PaywallUtil.PREMIUM_MONTHLY_PRICE_ID:
            print(f'found {account_id} is level_two with product_id:{stripe_subscription_product}')
            return SuplexedAccountLevel_Two(account_id=account_id, db_session=db_session, is_hydrate_all=False)            

        print(f'no account level could be found for account_id={account_id}, product={stripe_subscription_product}')
        raise Exception(f'Could not find account level for price_id={stripe_subscription_product}')


if __name__ == '__main__':
    init_db()
    db_session = get_session()
    account_id = '8dcf6c24-b0c3-11ea-a0e8-7eb4e3ce69c3'
    account_level = PaywallUtil.get_suplexed_account_level(account_id=account_id, db_session=db_session)

    print(f'active pricing strategies = {account_level.get_next_active_pricing_strategy_count()}')
    print(f'active listings = {account_level.get_next_listing_count()}')
    print(f'active synced_listings = {account_level.get_next_active_synced_count()}')
