import webbrowser
from pprint import pprint
from abc import ABC, abstractclassmethod
from typing import List

from suplexedlibrary.model.DataSyncAccount import DataSyncAccount
from suplexedlibrary.model.InventoryItem import InventoryItem
from suplexedlibrary.model.ResellPricing import ResellPricing
from suplexedlibrary.api.SuplexApi import SuplexApi
from suplexedlibrary.constants.PricerStrategy import PricerStrategy

def inventory_filter(inventory_item: InventoryItem) -> bool:
    ignore_statuses = {
        'return pending',
        'cancellation',
        'cancelled',
        'return approved'
    }

    if not inventory_item.pricingStrategy:
        return False

    if not inventory_item.resellPricing:
        print(f'no pricing data for {inventory_item.shoeSku}, {inventory_item.shoeSize}')
        return False

    return True


class ConsignmentPricer(ABC):

    def __init__(self, account: DataSyncAccount):
        self.account = account
        self.suplex_api = SuplexApi()
        self.is_listing_data_healthy = True

    def price_inventory(self):
        if not self.is_listing_data_healthy:
            print('We are skipping the pricer because we are not getting accurate listing data from the marketplace')
            return
        
        inventory_list = self.get_inventory()

        for inventory_item in inventory_list:
            print(f'pricing {inventory_item.shoeName}')
            try:
                if inventory_item.saleOrder:
                    # print(f'This inventory item {inventory_item.id} sold, and is still listed on the exchange, delete it')
                    print('this ask belongs to sold inventory, delete it')
                    # self.delete_exchange_ask(inventory_item)

                elif inventory_item.pricingStrategy.strategy_type == PricerStrategy.FLOOR:
                    # print('selling bottom')
                    print('selling bottom')
                    self.sell_bottom(inventory_item)

                elif inventory_item.pricingStrategy.strategy_type == PricerStrategy.LIQUIDATE:
                    # print('selling now')
                    print('selling now')
                    self.sell_now()

                elif inventory_item.pricingStrategy.strategy_type == PricerStrategy.STATIC:
                    # print('static sell')
                    print('static sell')
                    self.sell_static_price(inventory_item)
                else:
                    print('what is this?')
            except Exception as e:
                print(e)

    def sell_static_price(self, inventory_item: InventoryItem):
        # print('\tstatic pricing')
        print('static pricing')
        print(f'\tstatic pricing {inventory_item.shoeSku}')
        if self.has_active_listing_been_removed_from_the_marketplace(inventory_item):
            print(f'[({inventory_item.shoeSku}, {inventory_item.shoeSize}, {inventory_item.shoeName})]: this item has been deleted on the marketplace, sync on our side')
            return

        if self.does_ask_need_update(inventory_item):
            print('\tupdating this static ask price')
            print('\tupdating this static ask price')
            self.update_inventory_item_ask(inventory_item, self.get_ask_amount(inventory_item), is_floor_update=False)
        else:
            print('\tthis ask is good to go')
            print('\tthis ask is good to go')

    def sell_bottom(self, shoe: InventoryItem):
        print(f'Pricing {shoe.shoeName} at bottom market')
        print(f'Pricing {shoe.shoeName} at bottom market')

        if not shoe.resellPricing:
            print(f'No resell pricing found for {shoe.shoeSku}')

        lowest_ask = self.get_lowest_ask(shoe.resellPricing)
        print(f'\tbottom_market:{lowest_ask}')

        if self.is_lowest_ask_calculated(shoe.resellPricing) and shoe.resellPricing.goat_ask == 0:
            print(f'lowest ask of {shoe.shoeSku} | {shoe.shoeSize} | {lowest_ask} is calculated, continuing')
            return
        elif self.is_lowest_ask_calculated(shoe.resellPricing):
            print(f'running with a calculated ask of {lowest_ask} for {shoe.shoeSize} size={shoe.shoeSize}')

        if not self.is_resell_price_valid(shoe):
            return
            print('oh boy')

        if self.is_bottom_market_our_shoe(shoe, lowest_ask):
            print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: We have another sneaker at the bottom market skipping for now')
            print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: We have another sneaker at the bottom market skipping for now')
            return

        # We already own bottom market
        elif self.is_at_bottom_market(shoe, lowest_ask):
            print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: We are already at the bottom')
            print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: We are already at the bottom')
            return

        # Lowering this shoe would bring us below the floor, we dont want to do this
        elif self.is_at_floor(shoe):
            print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: Sneaker is already at the floor')
            print(f'\tLowest_ask={lowest_ask} | floor={self.get_ask_amount(shoe)}')
            print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: Sneaker is already at the floor')
            print(f'\tLowest_ask={lowest_ask} | floor={self.get_ask_amount(shoe)}')

        elif self.would_go_below_floor(shoe, lowest_ask - 1):
            print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: New ask would go below floor')
            print(f'\tLowest_ask={lowest_ask} | floor={self.get_ask_amount(shoe)}')
            print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: New ask would go below floor')
            print(f'\tLowest_ask={lowest_ask} | floor={self.get_ask_amount(shoe)}')

        # We have a live ask so we need to perform an update
        else:
            print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: Updating ask for {shoe.shoeSku}')
            print(f'\tLowest_ask={lowest_ask} | new_ask={lowest_ask - 1}')
            print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: Updating ask for {shoe.shoeSku}')
            print(f'\tLowest_ask={lowest_ask} | new_ask={lowest_ask - 1}')                
            self.update_inventory_item_ask(shoe, lowest_ask - 1, is_floor_update=True)


    def sell_now(self):
        pass

    def would_go_below_floor(self, shoe: InventoryItem, lowest_ask: int) -> bool:
        if lowest_ask < self.get_ask_amount(shoe):
            return True

        return False

    @abstractclassmethod
    def get_lowest_ask(cls, resell_pricing: ResellPricing) -> int:
        pass

    @abstractclassmethod
    def does_ask_need_update(cls, inventory_item: InventoryItem) -> bool:
        pass

    @abstractclassmethod
    def update_inventory_item_ask(cls, inventory_item: InventoryItem, new_ask: int, is_floor_update: bool):
        pass

    @abstractclassmethod
    def is_at_bottom_market(cls, inventory_item: InventoryItem, bottom_market: int) -> bool:
        pass

    @abstractclassmethod
    def is_at_floor(cls, inventory_item: InventoryItem) -> bool:
        pass

    @abstractclassmethod
    def is_bottom_market_our_shoe(cls, inventory_item: InventoryItem, bottom_market: int) -> bool:
        pass

    @abstractclassmethod
    def hydrate_ask_map(cls):
        pass

    @abstractclassmethod
    def get_ask_amount(cls):
        pass

    @abstractclassmethod
    def get_inventory(cls) -> List[InventoryItem]:
        pass

    @abstractclassmethod
    def is_lowest_ask_calculated(cls) -> bool:
        pass
    
    @abstractclassmethod
    def has_active_listing_been_removed_from_the_marketplace(cls) -> bool:
        pass

    @abstractclassmethod
    def is_resell_price_valid(cls, inventory_item: InventoryItem) -> bool:
        pass