import webbrowser
from pprint import pprint
from abc import ABC, abstractclassmethod
from typing import List

from suplexedlibrary.model.DataSyncAccount import DataSyncAccount
from suplexedlibrary.model.InventoryItem import InventoryItem
from suplexedlibrary.model.ResellPricing import ResellPricing
from suplexedlibrary.api.SuplexApi import SuplexApi
from suplexedlibrary.constants.PricerStrategy import PricerStrategy
from suplexedlibrary.database.postgres.postgres import Session



def inventory_filter(inventory_item: InventoryItem) -> bool:
    ignore_statuses = {
        'return pending',
        'cancellation',
        'cancelled',
        'return approved'
    }

    if not inventory_item.pricingStrategy:
        return False

    if not inventory_item.resellPricing:
        print(f'no pricing data for {inventory_item.shoeSku}, {inventory_item.shoeSize}')
        return False

    return True


class Pricer(ABC):

    def __init__(self, account: DataSyncAccount):
        self.account = account
        self.suplex_api = SuplexApi()
        self.is_listing_data_healthy = True


    def price_single_inventory_item(self, inventory_item_id: str):
        inventory_item = self.db_session.query(InventoryItem).filter(InventoryItem.id == inventory_item_id).first()
        print(f'pricing {inventory_item.shoeName}')
        try:
            if inventory_item.saleOrder:
                if self.is_listed_on_exchange(inventory_item):
                    # print(f'This inventory item {inventory_item.id} sold, and is still listed on the exchange, delete it')
                    print('this ask belongs to sold inventory, delete it')
                    print('This inventory item {inventory_item.id} sold, and is still listed on the exchange, delete it')
                    self.delete_exchange_ask(inventory_item)
                else:
                    print('this ask belongs to sold inventory but is not active on the marketplace')
                    print('This inventory item {inventory_item.id} sold and isn\'t on the marketplace, continue')

            elif self.is_listed_on_exchange(inventory_item) and (self.is_ask_inactive(inventory_item) or self.is_ask_deleted(inventory_item)):
                self.delete_exchange_ask(inventory_item)
            
            # elif self.is_listed_on_exchange(inventory_item) and self.is_ask_deleted(inventory_item):
            #     self.delete_inventory_item_ask(inventory_item)

            elif self.is_ask_inactive(inventory_item):
                print('This ask is inactive and not listed on the marketplace, we should skip it')
                # print('This ask is inactive and not listed on the marketplace, we should skip it')

            elif inventory_item.pricingStrategy.strategy_type == PricerStrategy.FLOOR:
                # print('selling bottom')
                print('selling bottom')
                self.sell_bottom(inventory_item)

            elif inventory_item.pricingStrategy.strategy_type == PricerStrategy.LIQUIDATE:
                # print('selling now')
                print('selling now')
                self.sell_now()

            elif inventory_item.pricingStrategy.strategy_type == PricerStrategy.STATIC:
                # print('static sell')
                print('static sell')
                self.sell_static_price(inventory_item)
            else:
                print('what is this?')
        except Exception as e:
            print('failed pricing an inventory item')
            print(e)



    def price_inventory(self):
        if not self.is_listing_data_healthy:
            print('We are skipping the pricer because we are not getting accurate listing data from the marketplace')
            return
        
        inventory_list = self.get_inventory()

        for inventory_item in inventory_list:
            self.price_single_inventory_item(inventory_item.id)

    def sell_static_price(self, inventory_item: InventoryItem):
        # print('\tstatic pricing')
        print('static pricing')
        if self.is_listed_on_exchange(inventory_item) and self.is_ask_inactive(inventory_item):
            print('removing inactivated ask from marketplace')
            self.delete_exchange_ask(inventory_item)

        elif self.is_ask_inactive(inventory_item):
            print('skipping ask as it is inactice')
            return
        if self.has_active_listing_been_removed_from_the_marketplace(inventory_item):
            print(f'[({inventory_item.shoeSku}, {inventory_item.shoeSize}, {inventory_item.shoeName})]: this item has been deleted on the marketplace, sync on our side')
            return
        elif self.is_listed_on_exchange(inventory_item):
            if self.does_ask_need_update(inventory_item):
                print('\tupdating this static ask price')
                self.update_inventory_item_ask(inventory_item, self.get_ask_amount(inventory_item), is_floor_update=False)
            else:
                print('\tthis ask is good to go')
        else:
            print('\tcreating a new marketplace listing for this ask')
            self.list_inventory_item_ask(inventory_item)

    def sell_bottom(self, shoe: InventoryItem):
        print(f'Pricing {shoe.shoeName} at bottom market')

        if not shoe.resellPricing:
            print(f'No resell pricing found for {shoe.shoeSku}')

        lowest_ask = self.get_lowest_ask(shoe.resellPricing)
        print(f'\tbottom_market:{lowest_ask}')

        if self.is_lowest_ask_calculated(shoe.resellPricing) or ((shoe.resellPricing.goat_ask == 0 and shoe.resellPricing.stockx_ask == 0) and self.account.email == 'daniel.revie1@gmail.com'):
            print(f'lowest ask of {shoe.shoeSku} | {shoe.shoeSize} | {lowest_ask} is calculated, continuing')
            return
        elif self.is_lowest_ask_calculated(shoe.resellPricing):
            print(f'running with a calculated ask of {lowest_ask} for {shoe.shoeSize} size={shoe.shoeSize}')

        if self.is_listing_expired(shoe):
            self.delete_exchange_ask(shoe)
            self.mark_listing_as_pending(shoe)
            self.sell_bottom(shoe)

        if self.is_listed_on_exchange(shoe):
            # print('\tupdating a new ask')

            if self.is_bottom_market_our_shoe(shoe, lowest_ask):
                print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: We have another sneaker at the bottom market skipping for now')
                return

            # We already own bottom market
            elif self.is_at_bottom_market(shoe, lowest_ask):
                print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: We are already at the bottom')
                return

            # Lowering this shoe would bring us below the floor, we dont want to do this
            elif self.is_at_floor(shoe):
                print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: Sneaker is already at the floor')
                print(f'\tLowest_ask={lowest_ask} | floor={self.get_ask_amount(shoe)}')

            elif self.would_go_below_floor(shoe, lowest_ask - 1):
                print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: New ask would go below floor')
                print(f'\tLowest_ask={lowest_ask} | floor={self.get_ask_amount(shoe)}')

            # We have a live ask so we need to perform an update
            else:
                print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: Updating ask for {shoe.shoeSku}')
                print(f'\tLowest_ask={lowest_ask} | new_ask={lowest_ask - 1}')                
                self.update_inventory_item_ask(shoe, lowest_ask - 1, is_floor_update=True)

        # Lets create a new ask to undercut the market, i.e. we dont have one yet
        else:
            print('\tcreating a new ask')
            if self.has_active_listing_been_removed_from_the_marketplace(shoe):
                print(f'[({shoe.shoeSku}, {shoe.shoeSize}, {shoe.shoeName})]: this item has been deleted on the marketplace, sync on our side')
                return

            elif self.is_bottom_market_our_shoe(shoe, lowest_ask):
                print('We have another matching shoe/size combo, matching asks')
                self.list_inventory_item_ask(shoe, lowest_ask)

            elif (lowest_ask - 1) <= self.get_ask_amount(shoe):
                print('Our floor is above the lowest ask, so placing at our floor')
                print(f'Lowest_ask={lowest_ask} | floor={self.get_ask_amount(shoe)}')
                self.list_inventory_item_ask(shoe, self.get_ask_amount(shoe))

            else:
                print(f'Placing a new ask at bottom market {lowest_ask - 1}')
                self.list_inventory_item_ask(shoe, lowest_ask - 1, self.get_start_price(shoe))

    def sell_now(self):
        pass

    def would_go_below_floor(self, shoe: InventoryItem, lowest_ask: int) -> bool:
        if lowest_ask < self.get_ask_amount(shoe):
            return True

        return False

    @abstractclassmethod
    def get_lowest_ask(cls, resell_pricing: ResellPricing) -> int:
        pass

    @abstractclassmethod
    def does_ask_need_update(cls, inventory_item: InventoryItem) -> bool:
        pass

    @abstractclassmethod
    def list_inventory_item_ask(cls, inventory_item: InventoryItem, ask_amount: int):
        pass

    @abstractclassmethod
    def update_inventory_item_ask(cls, inventory_item: InventoryItem, new_ask: int, is_floor_update: bool):
        pass

    @abstractclassmethod
    def delete_inventory_item_ask(cls, inventory_item: InventoryItem):
        pass

    @abstractclassmethod
    def is_at_bottom_market(cls, inventory_item: InventoryItem, bottom_market: int) -> bool:
        pass

    @abstractclassmethod
    def is_at_floor(cls, inventory_item: InventoryItem) -> bool:
        pass

    @abstractclassmethod
    def is_listed_on_exchange(cls, inventory_item: InventoryItem) -> bool:
        pass

    @abstractclassmethod
    def is_bottom_market_our_shoe(cls, inventory_item: InventoryItem, bottom_market: int) -> bool:
        pass

    @abstractclassmethod
    def remove_all_asks(cls):
        pass

    @abstractclassmethod
    def is_ask_inactive(cls):
        pass

    @abstractclassmethod
    def is_ask_deleted(cls):
        pass

    @abstractclassmethod
    def delete_exchange_ask(cls):
        pass

    @abstractclassmethod
    def hydrate_ask_map(cls):
        pass

    @abstractclassmethod
    def get_ask_amount(cls):
        pass

    @abstractclassmethod
    def get_inventory(cls) -> List[InventoryItem]:
        pass

    @abstractclassmethod
    def is_lowest_ask_calculated(cls) -> bool:
        pass
    
    @abstractclassmethod
    def has_active_listing_been_removed_from_the_marketplace(cls) -> bool:
        pass
    
    @abstractclassmethod
    def is_listing_expired(cls, inventory_item: InventoryItem) -> bool:
        pass

    @abstractclassmethod
    def mark_listing_as_pending(cls, inventory_item: InventoryItem):
        pass

    @abstractclassmethod
    def get_start_price(cls, inventory_item: InventoryItem) -> int:
        pass