from datetime import datetime
from typing import List
from collections import defaultdict
from sqlalchemy import or_
from suplexedlibrary.database.postgres.postgres import get_new_session
from suplexedlibrary.database.mongo.nosql import get_sneaker_db
from suplexedlibrary.model.DataSyncAccount import DataSyncAccount
from suplexedlibrary.model.InventoryItem import InventoryItem
from suplexedlibrary.model.ResellPricing import ResellPricing
from suplexedlibrary.model.AskGoat import AskGoat
from suplexedlibrary.model.AskStadiumGoods import AskStadiumGoods
from suplexedlibrary.model.AskFlightClub import AskFlightClub
from suplexedlibrary.model.AskStockX import AskStockX

db = get_sneaker_db()



''' Daily Inventory Value

- Generate total value
- Generate synced value
- Generate goat value
- Generate stadium goods value
- Generate stockx value
- Generate flight club value

'''

class ValueStats:
    goat_value = 0
    stockx_value = 0
    stadium_goods_value = 0
    flightclub_value = 0
    synced_value = 0
    total_value = 0
    
    def __init__(self, date: datetime):
        self.date = date

    def get_json(self):
        return {
            'goat_value': self.goat_value,
            'stockx_value': self.stockx_value,
            'stadium_goods_value': self.stadium_goods_value,
            'flightclub_value': self.flightclub_value,
            'synced_value': self.synced_value,
            'total_value': self.total_value,       
            'date': self.date
        }

def generate_daily_inventory_value(account: DataSyncAccount):
    db_session = get_new_session()
    inventory_cursor: List[InventoryItem] = db_session.query(InventoryItem).filter(
        InventoryItem.account_id == account.id,
        or_(
            InventoryItem.askFlightClub != None,
            InventoryItem.askStadiumGoods != None,
            InventoryItem.askStockX != None,
            InventoryItem.askGoat != None,
        )
    ).all()
    inventory_count = db_session.query(InventoryItem).filter(
        InventoryItem.account_id == account.id,
        or_(
            InventoryItem.askFlightClub != None,
            InventoryItem.askStadiumGoods != None,
            InventoryItem.askStockX != None,
            InventoryItem.askGoat != None,
        ),
        InventoryItem.soldDate == None
    ).count()
    today = datetime.now().date().today().strftime('%m/%d/%Y')
    value_stats = ValueStats(today)

    for i, inventory in enumerate(inventory_cursor):
        print(f'\tprocessing: [{i}/{inventory_count}] {inventory.shoeName}')
        resell_pricing: ResellPricing = inventory.resellPricing
        stockx_ask: AskStockX = inventory.askStockX
        goat_ask: AskGoat = inventory.askGoat
        stadium_goods_ask: AskStadiumGoods = inventory.askStadiumGoods
        flightclub_ask: AskFlightClub = inventory.askFlightClub

        if stockx_ask:
            value_stats.stockx_value += resell_pricing.stockx_ask if resell_pricing else 0

        if goat_ask:
            value_stats.goat_value += resell_pricing.goat_ask if resell_pricing else 0

        if stadium_goods_ask:
            value_stats.stadium_goods_value += stadium_goods_ask.ask_amount

        if flightclub_ask:
            value_stats.flightclub_value += flightclub_ask.ask_amount

        if inventory.synced:
            value_stats.synced_value += max(resell_pricing.stockx_ask if resell_pricing else 0, resell_pricing.goat_ask if resell_pricing else 0)

        value_stats.total_value += max(
            resell_pricing.stockx_ask if resell_pricing else 0, 
            resell_pricing.goat_ask if resell_pricing else 0,
            stadium_goods_ask.ask_amount if stadium_goods_ask else 0,
            flightclub_ask.ask_amount if flightclub_ask else 0
        )

    db.dailyValueStats.update_one(
        {'account_id': account.id, 'date': today},
        {'$set': value_stats.get_json()},
        upsert=True
    )



def process_accounts():
    db_session = get_new_session()

    account_cursor = db_session.query(DataSyncAccount).all()

    for account in account_cursor:
        print(f'processing {account.email}')
        generate_daily_inventory_value(account)


if __name__ == '__main__':
    process_accounts()