#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:copyright: (c) 2017 by Lev Lazinskiy
:license: MIT, see LICENSE for more details.
"""
import os
import sys

from setuptools import setup, find_packages
from setuptools.command.install import install


VERSION = "1.1.30"
USERNAME = 'hamsolo'
EMAIL = 'daniel.revie1@gmail.com'

class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = os.getenv('CIRCLE_TAG')

        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            sys.exit(info)

setup(
    name="green_island_core",
    version=VERSION,
    author=USERNAME,
    author_email=EMAIL,
    description="Core functions",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'pydantic',
        'alembic',
        'amqp',
        'aniso8601',
        'bcrypt',
        'billiard',
        'blinker',
        'boto3',
        'botocore',
        'bs4',
        'celery',
        'certifi',
        'cffi',
        'chardet',
        'click',
        'click-didyoumean',
        'click-repl',
        'cognitojwt',
        'colorama',
        'coverage',
        'Cython',
        'datadog',
        'ddtrace',
        'decorator',
        'dnspython',
        'ecdsa',
        'enum-compat',
        'eventlet',
        'Flask',
        'Flask-Bcrypt',
        'Flask-Cognito',
        'Flask-Cors',
        'Flask-Mail',
        'Flask-Migrate',
        'flask-restplus',
        'Flask-Script',
        'Flask-SQLAlchemy',
        'Flask-Testing',
        'greenlet',
        'gunicorn',
        'idna',
        'intervaltree',
        'itsdangerous',
        'Jinja2',
        'jmespath',
        'JSON-log-formatter',
        'jsonschema',
        'kombu',
        'Mako',
        'MarkupSafe',
        'prompt-toolkit',
        'protobuf',
        'psycopg2-binary',
        'pyasn1',
        'pycparser',
        'PyJWT',
        'pymongo',
        'python-dateutil',
        'python-editor',
        'python-jose',
        'python-json-logger',
        'pytz',
        'PyYAML',
        'redis',
        'requests',
        'rq',
        'rsa',
        's3transfer',
        'selenium',
        'six',
        'sortedcontainers',
        'soupsieve',
        'SQLAlchemy',
        'stripe',
        'tenacity',
        'urllib3',
        'vine',
        'wcwidth',
        'Werkzeug',
        'zipp',
        'algoliasearch',
        'cloudscraper',
        'fake-useragent',
        'statsd',
        'kafka-python'
    ],
    cmdclass={
        'verify': VerifyVersionCommand,
    }
)
